Type: Package
Package: tidytuesdayR
Title: Access the Weekly 'TidyTuesday' Project Dataset
Version: 1.3.2
Authors@R: c(
    person("Jon", "Harmon", , "jonthegeek@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-4781-4346")),
    person("Ellis", "Hughes", , "ellishughes@live.com", role = "aut"),
    person("Thomas", "Mock", , "j.thomasmock@gmail.com", role = "ctb"),
    person("Data Science Learning Community", , , "tidytuesday@dslc.io", role = "dtc")
  )
Description: 'TidyTuesday' is a project by the 'Data Science Learning
    Community' in which they post a weekly dataset in a public data
    repository (<https://github.com/rfordatascience/tidytuesday>) for
    people to analyze and visualize. This package provides the tools to
    easily download this data and the description of the source.
License: MIT + file LICENSE
URL: https://dslc-io.github.io/tidytuesdayR/,
        https://github.com/dslc-io/tidytuesdayR
BugReports: https://github.com/dslc-io/tidytuesdayR/issues
Depends: R (>= 4.1.0)
Imports: cli, gh, glue, jsonlite, lubridate (>= 1.7.0), magrittr, purrr
        (>= 1.0.0), readr (>= 1.0.0), rlang, rvest (>= 0.3.2), stbl (>=
        0.3.0), tidyr, tools (>= 3.1.0), utils, xml2 (>= 1.2.0)
Suggests: astgrepr, base64enc, covr, dplyr, fs, knitr, magick, openssl,
        readxl (>= 1.0.0), rmarkdown, rstudioapi (>= 0.2), stringr,
        testthat (>= 3.0.0), tibble, usethis, vctrs, withr, yaml
VignetteBuilder: knitr
Config/Needs/website: pkgdown
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-12 20:37:23 UTC; jonth
Author: Jon Harmon [aut, cre] (ORCID: <https://orcid.org/0000-0003-4781-4346>),
  Ellis Hughes [aut],
  Thomas Mock [ctb],
  Data Science Learning Community [dtc]
Maintainer: Jon Harmon <jonthegeek@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-12 20:50:02 UTC
Built: R 4.5.2; ; 2026-04-12 23:10:59 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,tt)
S3method(print,tt_data)
S3method(print,tt_dataset_table)
S3method(print,tt_dataset_table_list)
export("%>%")
export(last_tuesday)
export(readme)
export(tt_available)
export(tt_clean)
export(tt_curate_data)
export(tt_datasets)
export(tt_download)
export(tt_download_file)
export(tt_intro)
export(tt_load)
export(tt_load_gh)
export(tt_meta)
export(tt_save_dataset)
export(tt_submit)
export(use_tidytemplate)
importFrom(glue,glue)
importFrom(lubridate,today)
importFrom(magrittr,"%>%")
importFrom(stats,aggregate)
importFrom(stats,na.pass)
importFrom(stats,setNames)

# Paste code used to create the curated dataset here. Include comments as
# necessary. If you did not need to clean the data, use a comment like the one
# below, but also load the data with readr::read_csv() to ensure the data can be
# loaded, and to use with `saving.R`. Delete this block of comments.

# Clean data provided by <source of data>. No cleaning was necessary.
dataset <- readr::read_csv("dataset_url")

# Using vignette: https://dslc-io.github.io/tidytuesdayR/articles/curating.html
#
# Also available via vignette("curating", "tidytuesdayR")

# Load packages
library(tidytuesdayR)

# Run tt_clean() to create submission folder and cleaning script. The final
# result of the script should be one or more data frames, which we'll save in
# later steps. If the data is already clean, still provide a cleaning script in
# which you comment where the data came from and load it as a data frame.
tt_clean()

# Replace "your_dataset_name" with the name of the data frame created in
# tt_clean(). Run this once per data frame.
tt_save_dataset(your_dataset_name)

# Fill in data dictionary files. If you're using an IDE, they should be open.

# Create introduction to the dataset replacing comments inside "<!--" and "-->".
tt_intro()

# Save a .png image to the tt_submission folder. You might need to create one.

# Provide metadata about your submission.
#
# Example:
tt_meta(
  title = "The 50 US States",
  article_title = "U.S. Department of Commerce, Bureau of the Census",
  article_url = "https://www.census.gov/",
  source_title = "The R datasets package",
  source_url = "https://www.r-project.org/",
  image_filename = "states_population.png",
  image_alt = "A map of the continental United States, with each state colored in shades of blue by population as of 1975. California and New York are the lightest, indicating the highest population. Maine, New Hampshire, Vermont, and the Plains States are all quite dark, indicating low population.",
  attribution = "Jon Harmon, Data Science Learning Community",
  github = "jonthegeek",
  bluesky = "jonthegeek.com",
  linkedin = "jonthegeek",
  mastodon = "fosstodon.org/@jonthegeek"
)

# SAVE ALL YOUR FILES BEFORE SUBMITTING!!
tt_submit()

# If you make changes, run tt_submit() again to update the pull request.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## ----tt_curate_data-----------------------------------------------------------
# library(tidytuesdayR)
# tt_curate_data()

## ----tt_clean-----------------------------------------------------------------
# library(tidytuesdayR)
# tt_clean()

## ----cleaning-----------------------------------------------------------------
# library(tidyverse)
# library(janitor)
# 
# states <- state.x77 |>
#   tibble::as_tibble(rownames = "state") |>
#   janitor::clean_names() |>
#   dplyr::mutate(
#     dplyr::across(
#       c("population", "income", "frost", "area"),
#       as.integer
#     )
#   )

## ----tt_save_dataset----------------------------------------------------------
# tt_save_dataset(states)

## ----tt_intro-----------------------------------------------------------------
# tt_intro()

## ----states-population--------------------------------------------------------
# library(tidyverse)
# state_outlines <- ggplot2::map_data("state")
# states |>
#   dplyr::select(state, population) |>
#   dplyr::mutate(region = tolower(state)) |>
#   dplyr::left_join(state_outlines, by = "region") |>
#   ggplot(aes(long, lat, group = state, fill = population)) +
#   geom_polygon() +
#   coord_map() +
#   theme_void()
# ggsave(
#   usethis::proj_path("tt_submission", "states_population.png"),
#   width = 5, height = 3, units = "in",
#   bg = "white"
# )

## ----tt-meta------------------------------------------------------------------
# tt_meta(
#   title = "The 50 US States",
#   article_title = "U.S. Department of Commerce, Bureau of the Census",
#   article_url = "https://www.census.gov/",
#   source_title = "The R datasets package",
#   source_url = "https://www.r-project.org/",
#   image_filename = "states_population.png",
#   image_alt = "A map of the continental United States, with each state colored in shades of blue by population as of 1975. California and New York are the lightest, indicating the highest population. Maine, New Hampshire, Vermont, and the Plains States are all quite dark, indicating low population.",
#   attribution = "Jon Harmon, Data Science Learning Community",
#   github = "jonthegeek",
#   bluesky = "jonthegeek.com",
#   linkedin = "jonthegeek",
#   mastodon = "fosstodon.org/@jonthegeek"
# )

## ----tt_submit----------------------------------------------------------------
# tt_submit()

#' Get Master List of Files from TidyTuesday
#'
#' Import or update dataset from github that records the entire list of objects
#' from TidyTuesday
#'
#' @inheritParams gh_get
#' @param force force the update to occur even if the SHA matches
#'
#' @returns The tt master file, updated if necessary.
#' @keywords internal
tt_master_file <- function(force = FALSE, auth = gh::gh_token()) {
  if (should_update_tt_master_file(force, auth)) {
    content <- gh_get_csv("static/tt_data_type.csv")
    TT_MASTER_ENV$TT_MASTER_FILE <- content
  }
  return(TT_MASTER_ENV$TT_MASTER_FILE)
}

#' Decide whether to update the master file
#'
#' @inheritParams tt_master_file
#'
#' @returns Boolean indicating whether the master file should be updated.
#' @keywords internal
should_update_tt_master_file <- function(force = FALSE, auth = gh::gh_token()) {
  force ||
    !nrow(TT_MASTER_ENV$TT_MASTER_FILE) ||
    is.null(attr(TT_MASTER_ENV$TT_MASTER_FILE, ".sha")) ||
    gh_get_sha_in_folder(
      "static",
      "tt_data_type.csv",
      auth = auth
    ) !=
      attr(TT_MASTER_ENV$TT_MASTER_FILE, ".sha")
}

# The Master List of Files from TidyTuesday
TT_MASTER_ENV <- new.env()
TT_MASTER_ENV$TT_MASTER_FILE <- data.frame(
  Week = integer(0),
  ate = character(0),
  ear = integer(0),
  data_files = character(0),
  data_type = character(0),
  delim = character(0)
)

#' Download a TidyTuesday dataset file
#'
#' Download an actual data file from the TidyTuesday github repository.
#'
#' @inheritParams .shared-params
#' @param x Index or name of file to download.
#' @param ... Additional parameters to pass to the parsing functions. Note:
#'   These arguments will be passed for all filetypes.
#'
#' @returns tibble containing the contents of the file downloaded from git
#' @export
#'
#' @family tt_download_file
#'
#' @examplesIf interactive()
#' tt_gh <- tt_load_gh("2019-01-15")
#'
#' agencies <- tt_download_file(tt_gh, 1)
#' launches <- tt_download_file(tt_gh, "launches.csv")
tt_download_file <- function(tt, x, ..., auth = gh::gh_token()) {
  file_info <- attr(tt, ".files")
  tt_date <- attr(tt, ".date")
  call <- rlang::caller_env()
  target <- tryCatch(
    tt_subset_file_info(file_info, x),
    error = function(e) {
      .pkg_abort(
        "File {x} not found in the available files for {tt_date}.",
        "bad_index",
        call = call
      )
    }
  )
  gh_response <- tt_download_file_raw(tt_date, target)
  tt_parse_download(
    gh_response,
    data_type = file_info$data_type[file_info$data_files == target],
    delim = file_info$delim[file_info$data_files == target],
    ...
  )
}

tt_subset_file_info <- function(file_info, x) {
  data_files <- setNames(file_info$data_files, file_info$data_files)
  return(data_files[[x]])
}

tt_download_file_raw <- function(tt_date, target) {
  tt_year <- lubridate::year(tt_date)
  gh_response <- gh_get(file.path("data", tt_year, tt_date, target))
  if (gh_response$content == "") {
    gh_response <- gh_get_url(gh_response$git_url)
  }
  return(gh_response)
}

tt_parse_download <- function(gh_response, ..., data_type, delim = NA) {
  switch(
    data_type,
    "rds" = return(tt_parse_rds(gh_response, ...)), # nocov 3 examples
    "xls" = return(tt_parse_excel(gh_response, ...)), # nocov 0 examples
    "xlsx" = return(tt_parse_excel(gh_response, ...)),
    "vgz" = return(tt_parse_vgz(gh_response, ...)), # nocov 12 examples
    "zip" = return(tt_parse_zip(gh_response, data_type, delim, ...)) # nocov 3 examples
  )
  file_content <- gh_extract_text(gh_response)
  delim <- tt_guess_delim(delim, data_type)
  readr::read_delim(I(file_content), delim = delim, show_col_types = FALSE, ...)
}

tt_parse_excel <- function(gh_response, ...) {
  rlang::check_installed("readxl")
  tt_parse_file(gh_response, parser_fn = readxl::read_excel, ...)
}

tt_parse_file <- function(gh_response, parser_fn, ...) {
  file_content <- jsonlite::base64_dec(gh_response$content)
  temp_file <- tempfile()
  on.exit(unlink(temp_file))
  writeBin(file_content, temp_file)
  parser_fn(temp_file, ...)
}

tt_guess_delim <- function(delim, data_type) {
  if (!is.na(delim)) {
    return(delim)
  }
  switch(
    tolower(data_type),
    "csv" = ",", # nocov
    "tsv" = "\t",
    "," # nocov
  )
}

# nocov start
tt_parse_zip <- function(gh_response, data_type, delim, ...) {
  delim <- tt_guess_delim(delim, data_type)
  tt_parse_file(gh_response, readr::read_delim, delim = delim, ...)
}

tt_parse_rds <- function(gh_response, ...) {
  tt_parse_file(gh_response, parser_fn = readRDS, ...)
}

tt_parse_vgz <- function(gh_response, ...) {
  # These are csv.gz, so we save then parse.
  tt_parse_file(
    gh_response,
    parser_fn = readr::read_csv,
    show_col_types = FALSE,
    ...
  )
}
# nocov end

test_that(".pkg_abort() throws a classed tidytuesdayR error (#149)", {
  stbl::expect_pkg_error_snapshot(
    .pkg_abort("This is a test error.", "test_subclass"),
    "tidytuesdayR",
    "test_subclass"
  )
})

#' Load TidyTuesday data from Github
#'
#' Pulls the readme and URLs of the data from the TidyTuesday
#' github folder based on the date provided
#'
#' @inheritParams gh_get
#' @inheritParams .shared-params
#'
#' @returns A `tt` object. This contains the files available for the week,
#'  readme html, and the date of the TidyTuesday.
#' @export
#' @examplesIf interactive()
#' # check to make sure there are requests still available
#' if (rate_limit_check(quiet = TRUE) > 30) {
#'   tt_gh <- tt_load_gh("2019-01-15")
#'   ## readme attempts to open the readme for the weekly dataset
#'   readme(tt_gh)
#'
#'   agencies <- tt_download(
#'     tt_gh,
#'     files = "agencies.csv"
#'   )
#' }
tt_load_gh <- function(x, week = NULL, auth = gh::gh_token()) {
  # Need the date before we start messaging.
  tt_date <- tt_check_date(x, week, auth = auth)

  cli::cli_inform(
    "---- Compiling #TidyTuesday Information for {tt_date} ----"
  )

  tt_compilation <- tt_compile(tt_date)
  n_files <- NROW(tt_compilation$files)
  cli::cli_inform(
    "--- There {cli::qty(n_files)} {?are/is/are} {n_files} file{?s} available ---"
  )

  structure(
    tt_compilation$files$data_files,
    ".files" = tt_compilation$files,
    ".readme" = tt_compilation$readme,
    ".date" = tt_date,
    class = "tt"
  )
}

#' Get TidyTuesday readme and list of files and HTML based on the date
#'
#' @inheritParams gh_get
#' @param date date of TidyTuesday of interest
#'
#' @keywords internal
tt_compile <- function(date, auth = gh::gh_token()) {
  files <- tt_week_data_files(date, auth = auth)
  readme <- tt_week_readme_html(date, auth = auth)
  list(
    files = files,
    readme = readme
  )
}

tt_week_data_files <- function(date, auth = gh::gh_token()) {
  ttmf <- tt_master_file()
  ttmf[ttmf$Date == date, c("data_files", "data_type", "delim")]
}

tt_week_readme_html <- function(date, auth = gh::gh_token()) {
  gh_get_readme_html(
    file.path("data", lubridate::year(date), date),
    auth = auth
  )
}

#' @rdname printing
#' @export
print.tt <- function(x, ...) {
  message(
    "Available datasets in this TidyTuesday:\n\t",
    paste(attr(x, ".files")$data_files, "\n\t", collapse = "")
  )
  invisible(x)
}

test_that("tt_week_readme_html loads the proper path", {
  local_mocked_bindings(
    gh_get_readme_html = function(path, ...) {
      return(path)
    }
  )
  expect_identical(
    tt_week_readme_html("2025-01-01"),
    "data/2025/2025-01-01"
  )
})

test_that("tt_compile lists all files for the date", {
  local_tt_master_file()
  local_tt_week_readme_html()
  tt_c <- tt_compile("2019-01-15")
  expect_equal(
    tt_c$files$data_files,
    c("agencies.csv", "launches.csv")
  )
  expect_equal(
    tt_c$files$data_type,
    c("csv", "csv")
  )
  expect_equal(
    tt_c$files$delim,
    c(",", ",")
  )
  expect_true(
    !is.null(tt_c$readme)
  )
})

test_that("tt_compile returns NULL for missing readme's", {
  local_tt_master_file()
  local_tt_week_readme_html()
  expect_warning(
    {
      tt_c <- tt_compile("2018-04-02")
    },
    "No readme found",
    class = "tt-warning-no_readme"
  )

  expect_equal(
    tt_c$files$data_files,
    "us_avg_tuition.xlsx"
  )
  expect_equal(
    tt_c$files$data_type,
    "xlsx"
  )
  expect_true(
    is.na(tt_c$files$delim)
  )
  expect_true(
    is.null(tt_c$readme)
  )
})

test_that("tt_load_gh returns tt object when provided proper date", {
  local_tt_master_file()
  local_tt_week_readme_html()
  expect_message(
    {
      expect_message(
        {
          tt <- tt_load_gh("2019-01-15")
        },
        "Compiling #TidyTuesday"
      )
    },
    "There are 2 files"
  )
  expect_s3_class(tt, "tt")
  expect_equal(
    attr(tt, ".files")$data_files,
    c("agencies.csv", "launches.csv")
  )
})

test_that("tt_load_gh returns tt object when provided proper year and TT week number", {
  local_tt_master_file()
  local_tt_week_readme_html()
  expect_message(
    {
      expect_message(
        {
          tt <- tt_load_gh(2019, 3)
        },
        "Compiling #TidyTuesday"
      )
    },
    "There are 2 files"
  )
  expect_s3_class(tt, "tt")
  expect_equal(
    attr(tt, ".files")$data_files,
    c("agencies.csv", "launches.csv")
  )
})

test_that("tt_load_gh errors when incorrect date", {
  local_tt_master_file()
  stbl::expect_pkg_error_snapshot(
    tt_load_gh("2019-01-16"),
    "tidytuesdayR",
    "invalid_date"
  )
})

test_that("tt_load_gh returns list of object when no readme.md available", {
  local_tt_master_file()
  local_tt_week_readme_html()
  expect_message(
    {
      expect_message(
        {
          expect_warning(
            {
              tt <- tt_load_gh("2018-04-09")
            },
            class = "tt-warning-no_readme"
          )
        },
        "Compiling #TidyTuesday"
      )
    },
    "There is 1 file"
  )
  expect_s3_class(tt, "tt")
  expect_true(length(attr(tt, ".readme")) == 0) # object should not exist
})

test_that("tt_load_gh ignores extra files/diretory paths", {
  local_tt_master_file()
  local_tt_week_readme_html()
  expect_message(
    {
      expect_message(
        {
          tt_obj <- tt_load_gh("2019-04-02")
        },
        "Compiling #TidyTuesday"
      )
    },
    "There is 1 file"
  )
  expect_equal(length(tt_obj), 1)
  expect_equal(tt_obj[1], "bike_traffic.csv")

  expect_message(
    {
      expect_message(
        {
          tt_obj_2 <- tt_load_gh("2019-04-09")
        },
        "Compiling #TidyTuesday"
      )
    },
    "There are 3 files"
  )
  expect_equal(length(tt_obj_2), 3)
  expect_equal(
    tt_obj_2[1:3],
    c(
      "grand_slam_timeline.csv",
      "grand_slams.csv",
      "player_dob.csv"
    )
  )
})

test_that("tt_load_gh works with tsvs", {
  local_tt_master_file()
  local_tt_week_readme_html()
  expect_message(
    {
      expect_message(
        {
          tt_obj <- tt_load_gh("2020-04-21")
        },
        "Compiling #TidyTuesday"
      )
    },
    "There are 2 files"
  )

  expect_equal(length(tt_obj), 2)
  expect_equal(tt_obj[1:2], c("gdpr_text.tsv", "gdpr_violations.tsv"))
})

test_that("print.tt lists all the available files for the weeks tt", {
  local_tt_master_file()
  local_tt_week_readme_html()
  expect_message(expect_message({
    tt <- tt_load_gh("2019-01-15")
  }))
  expect_snapshot({
    test_result <- print(tt)
  })
  expect_identical(test_result, tt)
})

#' Create and open meta.yaml
#'
#' We need a set of metadata information about each TidyTuesday dataset. Use
#' this function to set up the `meta.yaml` file for your submission (and create
#' the submission directory if it does not already exist). If you do not provide
#' values for the parameters, you will be prompted to enter them in an
#' interactive session.
#'
#' @inheritParams usethis::use_template
#' @inheritParams .shared-params
#' @param title A short title for your submission. It should fit into the
#'   sentence "This week we're exploring `title`!" For example, for "This week
#'   we're exploring The 50 US States!", the `title` would be `"The 50 US
#'   States"`.
#' @param article_title The title of an article or other website that has
#'   something to do with the data. This should usually be an article that uses
#'   or describes the dataset, but any related website is acceptable.
#' @param article_url The URL of the article whose title is `article_title`.
#' @param source_title The title of the source of the dataset. This is usually a
#'   website, but might be an R package or a journal article, for example.
#' @param source_url A URL associated with the source. Ideally this should be a
#'   URL where users can download the data, but, if that isn't possible, provide
#'   a URL that is somehow related to the source of the data.
#' @param image_filename A character vector with at least one file name for an
#'   image to accompany the post. This might be a plot of the data, or some othe
#'   image somehow connected to the data.
#' @param image_alt Text that can take the place of the image for a visually
#'   impaired user or anybody else who cannot see the image. Don't just say "A
#'   plot of the data", but rather describe what information you can glean from
#'   the plot, such as "A map of the continental United States, with each state
#'   colored in shades of blue by population as of 1975. California and New York
#'   are the lightest, indicating the highest population. Maine, New Hampshire,
#'   Vermont, and the Plains States are all quite dark, indicating low
#'   population."
#' @param attribution Your name as you would like it to appear when we credit
#'   you in the post for this dataset. You can include a title and/or
#'   affiliation if you like, such as "Jon Harmon, Executive Director, Data
#'   Science Learning Community". Defaults to
#'   `getOption("tidytuesdayR.attribution")`. We recommend setting this option
#'   in your `.Rprofile` with `usethis::edit_r_profile()` so you don't have to
#'   provide it every time you submit a dataset.
#' @param github Your GitHub username, or a link to your profile on GitHub.
#' @param bluesky Your Bluesky username, or a link to your profile on Bluesky.
#'   Leave as `NULL` if you do not wish to be credited on Bluesky. Defaults to
#'   `getOption("tidytuesdayR.bluesky")`.
#' @param linkedin Your LinkedIn username, or a link to your profile on LinkedIn.
#'   Leave as `NULL` if you do not wish to be credited on LinkedIn. Defaults to
#'   `getOption("tidytuesdayR.linkedin")`.
#' @param mastodon Your mastodon server and username, or a link to your profile
#'   on a mastodon server. Leave as `NULL` if you do not wish to be credited on
#'   Mastodon. Defaults to `getOption("tidytuesdayR.mastodon")`.
#'
#' @returns A logical vector indicating whether the file was created or
#'   modified, invisibly.
#' @export
#'
#' @examplesIf interactive()
#'
#'   tt_meta()
tt_meta <- function(
  path = "tt_submission",
  title,
  article_title,
  article_url,
  source_title,
  source_url,
  image_filename,
  image_alt,
  attribution = getOption("tidytuesdayR.attribution"),
  github = gh::gh_whoami()$login,
  bluesky = getOption("tidytuesdayR.bluesky"),
  linkedin = getOption("tidytuesdayR.linkedin"),
  mastodon = getOption("tidytuesdayR.mastodon"),
  open = rlang::is_interactive(),
  ignore = FALSE
) {
  prep_tt_curate(path, ignore = ignore)
  meta_path <- fs::path(path, "meta.yaml")

  title <- ensure_arg_filled(
    rlang::maybe_missing(title),
    "What is the title of your submission?"
  )
  article_title <- ensure_arg_filled(
    rlang::maybe_missing(article_title),
    "What is the title of an article related to your submission?"
  )
  article_url <- ensure_arg_filled(
    rlang::maybe_missing(article_url),
    "What is the url of an article related to your submission?"
  )
  source_title <- ensure_arg_filled(
    rlang::maybe_missing(source_title),
    "What is the title of the source of the data in your submission?"
  )
  source_url <- ensure_arg_filled(
    rlang::maybe_missing(source_url),
    "What is the url of the source of the data your submission?"
  )
  images <- format_image_data(
    rlang::maybe_missing(image_filename),
    rlang::maybe_missing(image_alt)
  )
  attribution <- ensure_arg_filled(
    rlang::maybe_missing(attribution),
    "How would you like us to credit you in the post (your name and possibly affiliation)?"
  )
  social_media <- format_social_media(
    github = github,
    bluesky = bluesky,
    linkedin = linkedin,
    mastodon = mastodon
  )
  credits <- c(
    list(
      c(
        attribution_type = "post",
        attribution_text = attribution
      )
    ),
    social_media
  )

  usethis::use_template(
    "meta.yaml",
    save_as = meta_path,
    data = list(
      title = title,
      article_title = article_title,
      article_url = article_url,
      source_title = source_title,
      source_url = source_url,
      images = images,
      credits = credits
    ),
    ignore = ignore,
    open = open,
    package = "tidytuesdayR"
  )
}

ensure_arg_filled <- function(
  var,
  question,
  arg_name = rlang::caller_arg(var),
  call = rlang::caller_env()
) {
  arg_name <- remove_caller_arg_missing(arg_name)
  if (
    rlang::is_interactive() && (rlang::is_missing(var) || rlang::is_null(var))
  ) {
    # nocov start
    question <- paste0("\n", stringr::str_trim(question), "\n")
    cat(question)
    var <- readline()
  } # nocov end
  if (rlang::is_empty(var) || identical(var, "")) {
    .pkg_abort(
      "{.arg {arg_name}} is required.",
      "required_arg",
      call = call
    )
  }
  return(var)
}

format_image_data <- function(image_filename, image_alt) {
  image_filename <- ensure_arg_filled(
    rlang::maybe_missing(image_filename),
    "What is the image filename?"
  )
  image_alt <- ensure_arg_filled(
    rlang::maybe_missing(image_alt),
    "What text could serve in place of the image for a visually impaired person?"
  )
  if (length(image_filename) && length(image_alt) == length(image_filename)) {
    return(
      purrr::map2(image_filename, image_alt, function(file, alt) {
        list(image_filename = file, image_alt = alt)
      })
    )
  }
  .pkg_abort(
    "Please provide at least one image filename and corresponding alt text.",
    "image_data"
  )
}

format_social_media <- function(
  github = gh::gh_whoami()$login,
  bluesky = NULL,
  linkedin = NULL,
  mastodon = NULL
) {
  socials <- list(
    c(
      attribution_type = "github",
      attribution_text = format_social_name(github)
    ),
    c(
      attribution_type = "bluesky",
      attribution_text = format_social_name(bluesky)
    ),
    c(
      attribution_type = "linkedin",
      attribution_text = format_social_name(linkedin)
    ),
    c(
      attribution_type = "mastodon",
      attribution_text = format_mastodon(mastodon)
    )
  )
  return(purrr::keep(socials, \(social) {
    "attribution_text" %in%
      names(social) &&
      length(social[["attribution_text"]]) > 0 &&
      social[["attribution_text"]] != ""
  }))
}

format_social_name <- function(social) {
  if (!length(social)) {
    return(NULL)
  }
  username <- stringr::str_extract(social, "([^/@]+)(/)?$", group = 1)
  return(paste0("@", username))
}

format_mastodon <- function(mastodon) {
  # Mastodon might come in as a url (eg, https://fosstodon.org/@jonthegeek), as
  # username@server (eg, jonthegeek@fosstodon.org), or as @username@server (eg,
  # @jonthegeek@fosstodon.org). Any of these should format as @username@server.
  # (eg, @jonthegeek@fosstodon.org).
  if (!length(mastodon)) {
    return(NULL)
  }
  username <- NA
  server <- NA
  if (stringr::str_detect(mastodon, "/")) {
    username <- stringr::str_extract(mastodon, "([^/@]+)(/)?$", group = 1)
    server <- stringr::str_extract(mastodon, "(https://)?([^/]+)", group = 2)
  } else {
    username <- stringr::str_extract(mastodon, "([^@]+)@", group = 1)
    server <- stringr::str_extract(mastodon, "@([^@]+)$", group = 1)
  }
  if (is.na(username) || is.na(server)) {
    .pkg_abort(
      c(
        "Please provide a valid mastodon username and server.",
        i = "eg, yourname@yourserver.org or https://yourserver.org/@yourname"
      ),
      "mastodon_format"
    )
  }
  return(glue::glue("@{username}@{server}"))
}

test_that("tt_meta creates the expected file", {
  proj_dir <- withr::local_tempdir()
  full_submission_path <- fs::path(proj_dir, "tt_submission")
  usethis::local_project(proj_dir, force = TRUE, quiet = TRUE)
  test_result <- tt_meta(
    title = "The 50 US States",
    article_title = "U.S. Department of Commerce, Bureau of the Census",
    article_url = "https://www.census.gov/",
    source_title = "The R datasets package",
    source_url = "https://www.r-project.org/",
    image_filename = "states_population.png",
    image_alt = "A map of the continental United States, with each state colored in shades of blue by population as of 1975. California and New York are the lightest, indicating the highest population. Maine, New Hampshire, Vermont, and the Plains States are all quite dark, indicating low population.",
    attribution = "Jon Harmon, Data Science Learning Community",
    github = "jonthegeek",
    bluesky = "jonthegeek.com",
    linkedin = "jonthegeek",
    mastodon = "fosstodon.org/@jonthegeek"
  )
  expect_true(test_result)
  file_path <- fs::path(full_submission_path, "meta.yaml")
  expect_snapshot_file(file_path)
})

test_that("ensure_arg_filled errors informatively", {
  stbl::expect_pkg_error_snapshot(
    ensure_arg_filled("", question = "", arg_name = "myArg"),
    "tidytuesdayR",
    "required_arg"
  )
})

test_that("format_image_data errors informatively", {
  stbl::expect_pkg_error_snapshot(
    format_image_data(c("a", "b"), "c"),
    "tidytuesdayR",
    "image_data"
  )
})

test_that("format_social_name returns NULL for NULL", {
  expect_null(format_social_name(NULL))
})

test_that("format_mastodon returns NULL for NULL", {
  expect_null(format_mastodon(NULL))
})

test_that("format_mastodon extracts mastodon info from non-URLs", {
  expect_identical(
    format_mastodon("jonthegeek@fosstodon.org"),
    glue::glue("@jonthegeek@fosstodon.org")
  )
})

test_that("ensure_arg_filled errors informatively for NULL (#142)", {
  stbl::expect_pkg_error_snapshot(
    ensure_arg_filled(NULL, question = "", arg_name = "myArg"),
    "tidytuesdayR",
    "required_arg"
  )
})

test_that("tt_meta uses options for attribution fields (#142)", {
  proj_dir <- withr::local_tempdir()
  full_submission_path <- fs::path(proj_dir, "tt_submission")
  usethis::local_project(proj_dir, force = TRUE, quiet = TRUE)
  withr::local_options(
    tidytuesdayR.attribution = "Jon Harmon, Data Science Learning Community",
    tidytuesdayR.bluesky = "jonthegeek.com",
    tidytuesdayR.linkedin = "jonthegeek",
    tidytuesdayR.mastodon = "fosstodon.org/@jonthegeek"
  )
  test_result <- tt_meta(
    title = "The 50 US States",
    article_title = "U.S. Department of Commerce, Bureau of the Census",
    article_url = "https://www.census.gov/",
    source_title = "The R datasets package",
    source_url = "https://www.r-project.org/",
    image_filename = "states_population.png",
    image_alt = "A map of the continental United States, with each state colored in shades of blue by population as of 1975. California and New York are the lightest, indicating the highest population. Maine, New Hampshire, Vermont, and the Plains States are all quite dark, indicating low population.",
    github = "jonthegeek"
  )
  expect_true(test_result)
  file_path <- fs::path(full_submission_path, "meta.yaml")
  expect_snapshot_file(file_path, name = "meta-from-options.yaml")
})

#' Create and open intro.md
#'
#' When curating a TidyTuesday dataset, you need to introduce the dataset. This
#' function creates a simple `intro.md` file in the specified path (creating
#' that path if it does not already exist), and (if possible) opens it for
#' editing.
#'
#' @inheritParams usethis::use_template
#' @inheritParams .shared-params
#'
#' @returns A logical vector indicating whether the file was created or
#'   modified, invisibly.
#' @export
#'
#' @examplesIf interactive()
#'
#'   tt_intro()
tt_intro <- function(
  path = "tt_submission",
  open = rlang::is_interactive(),
  ignore = FALSE
) {
  prep_tt_curate(path, ignore = ignore)
  intro_path <- fs::path(path, "intro.md")

  usethis::use_template(
    "intro.md",
    save_as = intro_path,
    ignore = ignore,
    open = open,
    package = "tidytuesdayR"
  )
}

#' Create and open the tidytemplate
#'
#' Use the tidytemplate Rmd for starting your analysis with a leg up for
#' processing
#'
#' @inheritParams usethis::use_template
#' @param name A name for your generated TidyTuesday analysis Rmd, such as
#'   "My_TidyTuesday.Rmd".
#' @param refdate Date to use as reference to determine which TidyTuesday to use
#'   for the template. Either date object or character string in YYYY-MM-DD
#'   format.
#'
#' @returns A logical vector indicating whether the file was created or
#'   modified, invisibly.
#'
#' @examplesIf interactive()
#'
#'   use_tidytemplate(name = "My_Awesome_TidyTuesday.Rmd")
#'
#' @export
use_tidytemplate <- function(
  name = NULL,
  open = rlang::is_interactive(),
  refdate = today(),
  ignore = FALSE
) {
  rlang::check_installed("usethis", "to use the tidytemplate.")
  if (!valid_date(refdate)) {
    .pkg_abort(
      "{.arg refdate} must be a valid date.",
      "invalid_date"
    )
  }
  last_tt <- last_tuesday(refdate)
  if (is.null(name)) {
    name <- paste0(format(last_tt, "%Y_%m_%d"), "_tidy_tuesday.Rmd")
  }

  usethis::use_template(
    "tidytemplate.Rmd",
    save_as = name,
    data = list(
      call_date = today(),
      call_tuesday = format(last_tt, "%Y-%m-%d")
    ),
    package = "tidytuesdayR",
    ignore = ignore,
    open = open
  )
}

test_that("use_tidytemplate processes inputs", {
  local_mocked_bindings(
    use_template = function(...) {
      list(...)
    },
    .package = "usethis"
  )
  local_mocked_bindings(
    today = function() {
      as.Date("2024-08-19")
    }
  )
  expect_identical(
    use_tidytemplate(open = FALSE),
    list(
      "tidytemplate.Rmd",
      save_as = "2024_08_20_tidy_tuesday.Rmd",
      data = list(
        call_date = as.Date("2024-08-19"),
        call_tuesday = "2024-08-20"
      ),
      package = "tidytuesdayR",
      ignore = FALSE,
      open = FALSE
    )
  )
})

# Fixtures used in various tests, created by actually hitting APIs while online.
ttmf <- tt_master_file()
saveRDS(ttmf, test_path("fixtures", "ttmf.rds"))

static_contents <- gh_get("static") |>
  purrr::keep(~ .x$name %in% c("tt_data_type.csv", "tt_logo.png"))
saveRDS(static_contents, test_path("fixtures", "static_contents.rds"))
tt_data_type_response <- gh_get("static/tt_data_type.csv")
saveRDS(
  tt_data_type_response,
  test_path("fixtures", "tt_data_type_response.rds")
)
folder2020_response <- gh_get("data/2020")
saveRDS(folder2020_response, test_path("fixtures", "folder2020_response.rds"))
readme2020_response <- gh_get(
  "data/2020/readme.md",
  .accept = "application/vnd.github.v3.html"
)
saveRDS(readme2020_response, test_path("fixtures", "readme2020_response.rds"))

# Year readmes
readme2018 <- gh_get_readme_html(file.path("data", 2018))
xml2::write_html(readme2018, test_path("fixtures", "readme2018.html"))
readme2019 <- gh_get_readme_html(file.path("data", 2019))
xml2::write_html(readme2019, test_path("fixtures", "readme2019.html"))
readme2020 <- gh_get_readme_html(file.path("data", 2020))
xml2::write_html(readme2020, test_path("fixtures", "readme2020.html"))
readme2021 <- gh_get_readme_html(file.path("data", 2021))
xml2::write_html(readme2021, test_path("fixtures", "readme2021.html"))
readme2022 <- gh_get_readme_html(file.path("data", 2022))
xml2::write_html(readme2022, test_path("fixtures", "readme2022.html"))
readme2023 <- gh_get_readme_html(file.path("data", 2023))
xml2::write_html(readme2023, test_path("fixtures", "readme2023.html"))
readme2024 <- gh_get_readme_html(file.path("data", 2024))
xml2::write_html(readme2024, test_path("fixtures", "readme2024.html"))

# Week readmes
save_readme <- function(date) {
  tt_year <- lubridate::year(date)
  readme <- gh_get_readme_html(file.path("data", tt_year, date))
  xml2::write_html(
    readme,
    test_path("fixtures", glue::glue("readme{date}.html"))
  )
}
save_readme("2019-01-15")
save_readme("2019-04-02")
save_readme("2019-04-09")
save_readme("2020-04-21")
save_readme("2022-05-10")

# Files
save_file_response <- function(tt_date, target) {
  file <- glue::glue("response-{tt_date}-{target}.rds")
  path <- test_path("fixtures", file)
  saveRDS(
    tt_download_file_raw(tt_date, target),
    path
  )
}
save_file_response("2019-01-15", "agencies.csv")
save_file_response("2019-01-15", "launches.csv")
save_file_response("2018-04-02", "us_avg_tuition.xlsx")
save_file_response("2022-05-10", "nyt_titles.tsv")

# For this one, we only save the first version of the response.
tt_date <- "2018-05-29"
target <- "week9_comic_characters.csv"
tt_year <- lubridate::year(tt_date)
gh_response <- gh_get(file.path("data", tt_year, tt_date, target))
saveRDS(
  gh_response,
  test_path("fixtures", glue::glue("response-{tt_date}-{target}.rds"))
)

simple_tt_data <- structure(
  list(
    value1 = "value1",
    value2 = "value2"
  ),
  .tt = structure(
    c("value1.csv", "value2.csv"),
    .files = c("value1.csv", "value2.csv"),
    .readme = NULL,
    class = "tt_gh"
  ),
  class = "tt_data"
)
saveRDS(simple_tt_data, test_path("fixtures", "simple_tt_data.rds"))

#' print methods of the tt objects
#'
#' In tidytuesdayR there are nice print methods for the objects that were used
#' to download and store the data from the TidyTuesday repo. They will always
#'  print the available datasets/files. If there is a readme available,
#'  it will try to display the TidyTuesday readme.
#'
#' @name printing
#'
#' @inheritParams base::print
#' @param x a tt_data or tt object
#' @returns `x`, invisibly.
#'
#' @examplesIf interactive()
#' tt <- tt_load_gh("2019-01-15")
#' print(tt)
#'
#' tt_data <- tt_download(tt, files = "All")
#' print(tt_data)
NULL

#' Readme HTML maker and Viewer
#'
#' @param tt tt_data object for printing
#'
#' @returns Null, invisibly. Used to show readme of the downloaded TidyTuesday
#'   dataset in the Viewer.
#' @export
#' @examplesIf interactive()
#' if (rate_limit_check(quiet = TRUE) > 30) {
#'   tt_output <- tt_load_gh("2019-01-15")
#'   readme(tt_output)
#' }
readme <- function(tt) {
  if ("tt_data" %in% class(tt)) {
    tt <- attr(tt, ".tt")
  }
  if (length(attr(tt, ".readme")) > 0) {
    xml2::write_html(
      attr(tt, ".readme"),
      file = tmpHTML <-
        tempfile(fileext = ".html")
    )
    # if running in rstudio, print out that
    html_viewer(tmpHTML)
  }
  invisible(NULL)
}

html_viewer <- function(url, is_interactive = interactive()) {
  if (!is_interactive) {
    invisible(NULL)
  } else if (rstudio_is_available()) {
    rstudio_viewer(url = url)
  } else {
    browse_url(url = url)
  }
}

rstudio_is_available <- function() {
  rlang::is_installed("rstudioapi") && rstudioapi::isAvailable() # nocov
}

# For mocking in tests.
browse_url <- function(url) {
  utils::browseURL(url = url) # nocov
}

rstudio_viewer <- function(url) {
  rstudioapi::viewer(url = url) # nocov
}

contiguous_weeks <- function(week_vctr) {
  if (length(week_vctr) == 1) {
    text_out <- as.character(week_vctr)
  } else {
    is_not_contig <- which(diff(week_vctr) != 1)
    if (length(is_not_contig) == 0) {
      text_out <- paste0(week_vctr[1], "-", week_vctr[length(week_vctr)])
    } else {
      if (is_not_contig[[1]] == 1) {
        text_out <- as.character(week_vctr[1])
      } else {
        text_out <- paste0(week_vctr[1], "-", week_vctr[is_not_contig[[1]]])
      }
      contig_split <- 1
      while (contig_split < length(is_not_contig)) {
        if (
          diff(c(
            is_not_contig[contig_split],
            is_not_contig[contig_split + 1]
          )) ==
            1
        ) {
          text_out <- paste0(
            text_out,
            ", ",
            week_vctr[is_not_contig[contig_split] + 1]
          )
        } else {
          text_out <- paste0(
            text_out,
            ", ",
            paste0(
              week_vctr[is_not_contig[contig_split] + 1],
              "-",
              week_vctr[is_not_contig[contig_split + 1]]
            )
          )
        }
        contig_split %+=% 1
      }

      if (length(week_vctr) == (is_not_contig[contig_split] + 1)) {
        text_out <- paste0(
          text_out,
          ", ",
          week_vctr[length(week_vctr)]
        )
      } else {
        text_out <- paste0(
          text_out,
          ", ",
          paste0(
            week_vctr[is_not_contig[contig_split] + 1],
            "-",
            week_vctr[length(week_vctr)]
          )
        )
      }
    }
  }
  return(text_out)
}

`%+=%` <- function(x, y, env = parent.frame()) {
  x_name <- as.character(substitute(x))
  x_new <- x + y
  assign(
    x = x_name,
    value = x_new,
    envir = env
  )
}

remove_caller_arg_missing <- function(x) {
  stringr::str_remove_all(x, "(rlang::maybe_missing\\()|(\\))")
}

test_that("html_viewer returns NULL when running not in interactive mode", {
  res <- html_viewer("www.google.com", is_interactive = FALSE)
  expect_equal(res, NULL)
})

test_that("html_viewer works in interactive mode", {
  local_mocked_bindings(
    rstudio_is_available = function() {
      return(TRUE)
    },
    rstudio_viewer = function(url) {
      return("viewer")
    },
    browse_url = function(url) {
      return("browse_url")
    }
  )
  expect_equal(
    html_viewer("www.google.com", is_interactive = TRUE),
    "viewer"
  )

  local_mocked_bindings(
    rstudio_is_available = function() {
      return(FALSE)
    }
  )
  expect_equal(
    html_viewer("www.google.com", is_interactive = TRUE),
    "browse_url"
  )
})


test_that("readme() will attempt to display the contents of the readme attribute", {
  tt_data <- structure(
    list(
      value1 = "value1",
      value2 = "value2"
    ),
    .tt = structure(
      c("value1.csv", "value2.csv"),
      .files = c("value1.csv", "value2.csv"),
      .readme = xml2::read_html("<body><p>hello world</p></body>"),
      class = "tt_gh"
    ),
    class = "tt_data"
  )

  existing_files <- list.files(tempdir())
  readme(tt_data)
  readme_html <- setdiff(list.files(tempdir()), existing_files)

  expect_equal(
    xml2::read_html(file.path(tempdir(), readme_html)),
    attr(attr(tt_data, ".tt"), ".readme")
  )
})


test_that("contiguous_weeks() will attempt to display the contiguous weeks, collapsing continuous weeks", {
  expect_equal(contiguous_weeks(1), "1")

  expect_equal(contiguous_weeks(c(1, 2)), "1-2")

  expect_equal(contiguous_weeks(c(1:5)), "1-5")

  expect_equal(contiguous_weeks(c(1:5, 7)), "1-5, 7")

  expect_equal(contiguous_weeks(c(1:5, 7, 8)), "1-5, 7-8")

  expect_equal(contiguous_weeks(c(1:5, 7, 9)), "1-5, 7, 9")

  expect_equal(contiguous_weeks(c(1:5, 7, 9:11)), "1-5, 7, 9-11")

  expect_equal(contiguous_weeks(c(1:5, 7, 9:11, 15)), "1-5, 7, 9-11, 15")

  expect_equal(contiguous_weeks(c(5, 7, 9:11, 15)), "5, 7, 9-11, 15")

  expect_equal(contiguous_weeks(c(5, 7, 9:11, 15:100)), "5, 7, 9-11, 15-100")
})

#' Parameters used in multiple functions
#'
#' Reused parameter definitions are gathered here for easier editing.
#'
#' @param auth A GitHub token. See [gh::gh_token()] for more details.
#' @param call (`environment`) The caller environment for error messages.
#' @param files Which file names to download. Default "All" downloads all files
#'   for the specified week.
#' @param path The relative path to the directory to hold your submission files
#'   (`tt_submission` by default). If this directory does not exist, it will be
#'   created.
#' @param tt A `tt` object, output from [tt_load_gh()].
#' @param week Which week number to use within a given year. Only used when `x`
#'   is a valid year.
#' @param x The date of data to pull (in "YYYY-MM-dd" format), or the four-digit
#'   year as a number.
#' @param year What year of TidyTuesday to use
#'
#' @name .shared-params
#' @keywords internal
NULL

#' Get date of TidyTuesday, given the year and week
#'
#' Sometimes we don't know the date we want, but we do know the week. This
#' function provides the ability to pass the year and week we are interested in
#' to get the correct date
#'
#' @inheritParams gh_get
#' @inheritParams .shared-params
#' @keywords internal
tt_date <- function(year, week = NULL, auth = gh::gh_token()) {
  tt_check_date(year, week, auth = auth)
}

#' Generate valid TidyTuesday URL
#'
#' Given multiple types of inputs, generate a valid TidyTuesday URL.
#'
#' @inheritParams gh_get
#' @inheritParams .shared-params
#' @keywords internal
tt_check_date <- function(x, week = NULL, auth = gh::gh_token()) {
  if (missing(x)) {
    .pkg_abort(
      "Provide either the year & week or the date of the TidyTuesday dataset.",
      "invalid_date"
    )
  }

  if (valid_date(x)) {
    tt_check_date.date(x, auth = auth)
  } else if (valid_year(x)) {
    tt_check_date.year(x, week, auth = auth)
  } else {
    .pkg_abort("Entries must render to a valid date or year", "invalid_date")
  }
}

tt_check_date.date <- function(x, auth = gh::gh_token()) {
  tt_year <- lubridate::year(x)
  tt_formatted_date <- tt_date_format(x)
  if (as.character(tt_formatted_date) %in% c("2018-05-15", "2018-05-21")) {
    .pkg_abort(
      "The dataset for {tt_formatted_date} is dirty and cannot be automatically loaded.",
      "invalid_date"
    )
  }

  tt_folders <- tt_weeks(tt_year, auth = auth)
  if (!as.character(tt_formatted_date) %in% tt_folders[["folders"]]) {
    closest <- tt_closest_date(tt_formatted_date, tt_folders$folders)
    .pkg_abort(
      c(
        "{tt_formatted_date} does not have TidyTuesday data.",
        i = "Did you mean {closest}?"
      ),
      "invalid_date"
    )
  }
  tt_formatted_date
}

tt_check_date.year <- function(x, week, auth = gh::gh_token()) {
  if (x == 2018 && week %in% c(7, 8)) {
    .pkg_abort(
      "The dataset for 2018 week {week} is dirty and cannot be automatically loaded.",
      "invalid_date"
    )
  }

  tt_folders <- tt_weeks(x, auth = auth)

  if (!week %in% tt_folders$week_desc && week >= 1) {
    weeks <- contiguous_weeks(tt_folders$week_desc)
    .pkg_abort(
      c(
        "Week {week} does not have TidyTuesday data in {x}.",
        i = "Please choose a valid week from {weeks}"
      ),
      "invalid_date"
    )
  } else if (week < 1) {
    .pkg_abort(
      c(
        "{.arg week} must be a valid positive integer value."
      ),
      "invalid_date"
    )
  }

  tt_date <- tt_folders$folders[tt_folders$week_desc == week]

  if (
    !tt_date %in% tt_folders[["folders"]] ||
      !tt_folders[["data"]][tt_folders[["folders"]] == tt_date]
  ) {
    .pkg_abort(
      "Week {week} of {x} does not have data available for download.",
      "invalid_date"
    )
  }

  tt_date_format(tt_date)
}

tt_check_year <- function(year, auth = gh::gh_token()) {
  tt_yrs <- tt_years(auth = auth)
  if (year %in% tt_yrs) {
    return(invisible(year))
  }
  .pkg_abort(
    c(
      "TidyTuesday did not exist in {year} (or {year} is in the future).",
      i = "Available years: {tt_yrs}"
    ),
    "invalid_year"
  )
}


tt_weeks <- function(year, auth = gh::gh_token()) {
  tt_check_year(year, auth = auth)

  ttmf <- tt_master_file(auth = auth)

  tt_week <- aggregate(
    data_files ~ Week + Date,
    ttmf[ttmf$year == year, ],
    FUN = function(x) {
      !anyNA(x)
    },
    na.action = na.pass
  )

  list(
    week_desc = tt_week$Week,
    folders = tt_week$Date,
    data = tt_week$data_files
  )
}

tt_years <- function(auth = gh::gh_token()) {
  unique(tt_master_file(auth = auth)$year)
}

valid_date <- function(x) {
  suppressWarnings({
    lubridate::is.Date(x) || !is.na(lubridate::as_date(as.character(x)))
  })
}

valid_year <- function(x) {
  suppressWarnings({
    !is.na(lubridate::as_date(paste0(as.character(x), "-01-01")))
  })
}

tt_date_format <- function(x) {
  lubridate::ymd(paste0(
    lubridate::year(x),
    "-",
    lubridate::month(x),
    "-",
    lubridate::day(x)
  ))
}

tt_closest_date <- function(inputdate, availabledates) {
  availabledates[
    which.min(abs(difftime(inputdate, availabledates, units = "days")))
  ]
}

test_that("valid dates work", {
  local_tt_master_file()
  tt_date <- tt_check_date("2019-04-02")
  expect_equal(
    tt_date,
    as.Date("2019-04-02")
  )
})

test_that("valid year-week combinations work", {
  local_tt_master_file()
  tt_date_1 <- tt_check_date(2019, 14)
  tt_date_2 <- tt_check_date("2019", 14)

  expect_equal(tt_date_1, as.Date("2019-04-02"))
  expect_equal(tt_date_2, as.Date("2019-04-02"))
})

test_that("Close dates are suggested if provided date is incorrect", {
  local_tt_master_file()
  stbl::expect_pkg_error_snapshot(
    tt_check_date("2019-04-04"),
    "tidytuesdayR",
    "invalid_date"
  )
})

test_that("Invalid weeks throw errors", {
  local_tt_master_file()
  stbl::expect_pkg_error_snapshot(
    tt_check_date(2018, 20),
    "tidytuesdayR",
    "invalid_date"
  )
  stbl::expect_pkg_error_snapshot(
    tt_check_date(2018, 0),
    "tidytuesdayR",
    "invalid_date"
  )
  stbl::expect_pkg_error_snapshot(
    tt_check_date(2018, 7),
    "tidytuesdayR",
    "invalid_date"
  )
  stbl::expect_pkg_error_snapshot(
    tt_check_date(2018, 8),
    "tidytuesdayR",
    "invalid_date"
  )
  stbl::expect_pkg_error_snapshot(
    tt_check_date(2020, 1),
    "tidytuesdayR",
    "invalid_date"
  )
})

test_that("invalid entries are flagged", {
  local_tt_master_file()
  stbl::expect_pkg_error_snapshot(
    tt_check_date("xyz"),
    "tidytuesdayR",
    "invalid_date"
  )
})

test_that("tt_check_year checks years", {
  local_tt_master_file()
  stbl::expect_pkg_error_snapshot(
    tt_check_year(2015),
    "tidytuesdayR",
    "invalid_year"
  )
})

test_that("tt_date also works", {
  # This is mostly just a wrapper around tt_check_date(), so most tests are
  # handled above.
  local_tt_master_file()
  tt_date_1 <- tt_date(2019, 14)
  tt_date_2 <- tt_date("2019", 14)

  expect_equal(tt_date_1, as.Date("2019-04-02"))
  expect_equal(tt_date_2, as.Date("2019-04-02"))
})

test_that("tt_check_date errors informatively with no args", {
  local_tt_master_file()
  stbl::expect_pkg_error_snapshot(
    tt_check_date(),
    "tidytuesdayR",
    "invalid_date"
  )
})

test_that("tt_check_date errors informatively for the dirtiest dataset", {
  local_tt_master_file()
  stbl::expect_pkg_error_snapshot(
    tt_check_date("2018-05-15"),
    "tidytuesdayR",
    "invalid_date"
  )
})

#' Load TidyTuesday data from Github
#'
#' @inheritParams .shared-params
#' @inheritParams tt_download
#'
#' @returns `tt_data` object, which contains data that can be accessed via `$`,
#'   and the readme for the week's TidyTuesday, which can be viewed by printing
#'   the object or calling [readme()].
#'
#' @examplesIf interactive()
#' tt_output <- tt_load("2019-01-15")
#' tt_output
#' agencies <- tt_output$agencies
#'
#' @export
tt_load <- function(x, week = NULL, files = "All", ..., auth = gh::gh_token()) {
  # download readme and identify files
  tt <- tt_load_gh(x, week, auth = auth)

  # download files
  tt_data <- tt_download(tt, files = files, ..., auth = auth)

  ## return tt_data object
  structure(
    tt_data,
    ".tt" = tt,
    class = "tt_data"
  )
}

#' @rdname printing
#' @export
print.tt_data <- function(x, ...) {
  readme(x)
  message(
    "Available datasets:\n\t",
    paste(tools::file_path_sans_ext(names(x)), "\n\t", collapse = "")
  )
  invisible(x)
}

#' Create and open cleaning.R
#'
#' The first step of curating a TidyTuesday dataset is cleaning the data. This
#' function creates a simple `cleaning.R` file in the specified path (creating
#' that path if it does not already exist), and (if possible) opens it for
#' editing.
#'
#' @inheritParams usethis::use_template
#' @inheritParams .shared-params
#'
#' @returns A logical vector indicating whether the file was created or
#'   modified, invisibly.
#' @export
#'
#' @examplesIf interactive()
#'
#'   tt_clean()
tt_clean <- function(
  path = "tt_submission",
  open = rlang::is_interactive(),
  ignore = FALSE
) {
  prep_tt_curate(path, ignore = ignore)
  cleaning_path <- fs::path(path, "cleaning.R")

  usethis::use_template(
    "cleaning.R",
    save_as = cleaning_path,
    ignore = ignore,
    open = open,
    package = "tidytuesdayR"
  )
}

test_that("tt_clean creates the expected file", {
  proj_dir <- withr::local_tempdir()
  full_submission_path <- fs::path(proj_dir, "tt_submission")
  usethis::local_project(proj_dir, force = TRUE, quiet = TRUE)
  expect_true(tt_clean(open = FALSE))
  file_path <- fs::path(full_submission_path, "cleaning.R")
  expect_snapshot_file(file_path)
})

tt_enable_covr <- function() {
  options("tidytuesdayR.covr_check" = TRUE)
}

tt_disable_covr <- function() {
  options("tidytuesdayR.covr_check" = FALSE)
}

tt_is_checking_covr <- function() {
  return(getOption("tidytuesdayR.covr_check", FALSE))
}

local_tt_mocked_bindings <- function(..., .env = parent.frame()) {
  # Things that use mocks aren't reporting coverage properly.
  if (tt_is_checking_covr()) {
    return(invisible())
  }
  local_mocked_bindings(..., .env = .env)
}

local_tt_master_file <- function(.env = parent.frame()) {
  local_tt_mocked_bindings(
    tt_master_file = function(...) {
      return(readRDS(test_path("fixtures", "ttmf.rds")))
    },
    .env = .env
  )
}

local_tt_datasets <- function(.env = parent.frame()) {
  local_tt_mocked_bindings(
    gh_get_readme_html = function(path, auth = NULL) {
      year <- basename(path)
      file <- glue::glue("readme{year}.html")
      return(xml2::read_html(test_path("fixtures", file)))
    },
    .env = .env
  )
}

local_tt_week_readme_html <- function(.env = parent.frame()) {
  local_tt_mocked_bindings(
    tt_week_readme_html = function(date, auth = NULL) {
      file <- glue::glue("readme{date}.html")
      path <- test_path("fixtures", file)
      if (file.exists(path)) {
        return(xml2::read_html(path))
      }
      cli::cli_warn(
        "No readme found in path.",
        class = "tt-warning-no_readme"
      )
      return(NULL)
    },
    .env = .env
  )
}

local_tt_download_file_raw <- function(.env = parent.frame()) {
  local_tt_mocked_bindings(
    tt_download_file_raw = function(tt_date, target) {
      file <- glue::glue("response-{tt_date}-{target}.rds")
      path <- test_path("fixtures", file)
      if (file.exists(path)) {
        return(readRDS(path))
      }
      cli::cli_abort("Test file not found.")
    },
    .env = .env
  )
}

local_gh_get <- function(.env = parent.frame()) {
  local_tt_mocked_bindings(
    gh_get = function(path, ...) {
      file <- stringr::str_extract(path, "\\d{4}-\\d{2}-\\d{2}/.+$")
      file <- stringr::str_replace_all(file, "/", "-")
      file <- glue::glue("response-{file}.rds")
      path <- test_path("fixtures", file)
      if (file.exists(path)) {
        return(readRDS(path))
      }
      cli::cli_abort("Test file not found.")
    },
    .env = .env
  )
}

local_gh_get_sha_in_folder <- function(.env = parent.frame()) {
  local_tt_mocked_bindings(
    gh_get_sha_in_folder = function(path, file, auth) {
      return("5b7d51181d18d1af90caedd4e008509722612efb")
    },
    .env = .env
  )
}

local_gh_get_csv_data_type <- function(.env = parent.frame()) {
  local_tt_mocked_bindings(
    gh_get_csv = function(path, ...) {
      if (path == "static/tt_data_type.csv") {
        return(
          gh_extract_csv(
            readRDS(test_path("fixtures", "tt_data_type_response.rds"))
          )
        )
      } else {
        cli::cli_abort("Test broken.")
      }
    },
    .env = .env
  )
}

local_readme <- function(.env = parent.frame()) {
  local_tt_mocked_bindings(
    readme = function(tt) {
      return(NULL)
    },
    .env = .env
  )
}

test_that("tt_download_file errors for bad index", {
  local_tt_master_file()
  local_tt_week_readme_html()
  expect_message(expect_message({
    tt <- tt_load_gh("2019-01-15")
  }))
  stbl::expect_pkg_error_snapshot(
    tt_download_file(tt, 3),
    "tidytuesdayR",
    "bad_index"
  )
  stbl::expect_pkg_error_snapshot(
    tt_download_file(tt, "bad_filename"),
    "tidytuesdayR",
    "bad_index"
  )
})

test_that("tt_download_file works for a valid tt", {
  local_tt_master_file()
  local_tt_week_readme_html()
  local_tt_download_file_raw()
  expect_message(expect_message({
    tt <- tt_load_gh("2019-01-15")
  }))
  test_result <- tt_download_file(tt, 1)
  expect_s3_class(test_result, c("tbl_df", "data.frame"))
  expect_setequal(
    colnames(test_result),
    c(
      "agency",
      "count",
      "ucode",
      "state_code",
      "type",
      "class",
      "tstart",
      "tstop",
      "short_name",
      "name",
      "location",
      "longitude",
      "latitude",
      "error",
      "parent",
      "short_english_name",
      "english_name",
      "unicode_name",
      "agency_type"
    )
  )
})

test_that("tt_download_file works when delim isn't explicitly provided", {
  local_tt_master_file()
  local_tt_week_readme_html()
  local_tt_download_file_raw()
  expect_message(expect_message({
    tt <- tt_load_gh("2022-05-10")
  }))
  test_result <- tt_download_file(tt, "nyt_titles.tsv")
  expect_s3_class(test_result, c("tbl_df", "data.frame"))
  expect_setequal(
    colnames(test_result),
    c(
      "id",
      "title",
      "author",
      "year",
      "total_weeks",
      "first_week",
      "debut_rank",
      "best_rank"
    )
  )
})

test_that("tt_download_file downloads and parses xlsx", {
  local_tt_master_file()
  local_tt_week_readme_html()
  local_tt_download_file_raw()
  expect_message(expect_message(expect_warning({
    tt <- tt_load_gh("2018-04-02")
  })))
  test_result <- tt_download_file(tt, 1)
  expect_s3_class(test_result, c("tbl_df", "data.frame"))
  expect_setequal(
    colnames(test_result),
    c(
      "State",
      "2004-05",
      "2005-06",
      "2006-07",
      "2007-08",
      "2008-09",
      "2009-10",
      "2010-11",
      "2011-12",
      "2012-13",
      "2013-14",
      "2014-15",
      "2015-16"
    )
  )
})

test_that("tt_download_file downloads and parses large csv", {
  # This test kind of sucks but it's rare and this at least tests that the path
  # is followed.
  local_mocked_bindings(
    gh_get_url = function(url) {
      cli::cli_inform("downloading {url}")
    }
  )
  local_gh_get()
  expect_message(
    tt_download_file_raw("2018-05-29", "week9_comic_characters.csv"),
    "d183c7b0bec0b78409f063f3b80b0def2dfbe5c9"
  )
})

# test_that("tt_download_file downloads and parses rds", {
#   local_tt_master_file()
#   local_tt_week_readme_html()
#   local_tt_download_file_raw()
#   expect_message(expect_message(expect_warning(
#     {tt <- tt_load_gh("2018-04-02")}
#   )))
#   test_result <- tt_download_file(tt, 1)
#   expect_s3_class(test_result, c("tbl_df", "data.frame"))
#   expect_setequal(
#     colnames(test_result),
#     c(
#       "State", "2004-05", "2005-06", "2006-07", "2007-08", "2008-09", "2009-10",
#       "2010-11", "2011-12", "2012-13", "2013-14", "2014-15", "2015-16"
#     )
#   )
# })

# test_that("tt_download_file downloads and parses xls", {
#   local_tt_master_file()
#   local_tt_week_readme_html()
#   local_tt_download_file_raw()
#   expect_message(expect_message(expect_warning(
#     {tt <- tt_load_gh("2018-04-02")}
#   )))
#   test_result <- tt_download_file(tt, 1)
#   expect_s3_class(test_result, c("tbl_df", "data.frame"))
#   expect_setequal(
#     colnames(test_result),
#     c(
#       "State", "2004-05", "2005-06", "2006-07", "2007-08", "2008-09", "2009-10",
#       "2010-11", "2011-12", "2012-13", "2013-14", "2014-15", "2015-16"
#     )
#   )
# })

# test_that("tt_download_file downloads and parses vgz", {
#   local_tt_master_file()
#   local_tt_week_readme_html()
#   local_tt_download_file_raw()
#   expect_message(expect_message(expect_warning(
#     {tt <- tt_load_gh("2018-04-02")}
#   )))
#   test_result <- tt_download_file(tt, 1)
#   expect_s3_class(test_result, c("tbl_df", "data.frame"))
#   expect_setequal(
#     colnames(test_result),
#     c(
#       "State", "2004-05", "2005-06", "2006-07", "2007-08", "2008-09", "2009-10",
#       "2010-11", "2011-12", "2012-13", "2013-14", "2014-15", "2015-16"
#     )
#   )
# })

test_that("tt_download errors for bad file", {
  local_tt_master_file()
  local_tt_week_readme_html()
  expect_message(expect_message({
    tt <- tt_load_gh("2019-01-15")
  }))
  stbl::expect_pkg_error_snapshot(
    tt_download(tt, "bad_filename"),
    "tidytuesdayR",
    "bad_file"
  )
})

test_that("tt_download downloads all files", {
  local_tt_master_file()
  local_tt_week_readme_html()
  local_tt_download_file_raw()
  expect_message(expect_message({
    tt <- tt_load_gh("2019-01-15")
  }))
  # Extract expect_message level to catch newline.
  expect_message(expect_message(
    expect_message(
      expect_message(
        {
          tt_data <- tt_download(tt)
        },
        "Downloading files"
      ),
      "1 of 2",
      class = "tt-message-download"
    ),
    "2 of 2",
    class = "tt-message-download"
  ))
  expect_type(tt_data, "list")
  expect_length(tt_data, 2)
  expect_named(tt_data, c("agencies", "launches"))
  expect_s3_class(tt_data$agencies, "tbl_df")
  expect_s3_class(tt_data$launches, "tbl_df")
})

test_that("tt_download downloads specific files", {
  local_tt_master_file()
  local_tt_week_readme_html()
  local_tt_download_file_raw()
  expect_message(expect_message({
    tt <- tt_load_gh("2019-01-15")
  }))
  # Extract expect_message level to catch newline.
  expect_message(expect_message(
    expect_message(
      {
        tt_data <- tt_download(tt, files = "agencies.csv")
      },
      "Downloading files"
    ),
    "1 of 1",
    class = "tt-message-download"
  ))
  expect_type(tt_data, "list")
  expect_length(tt_data, 1)
  expect_s3_class(tt_data$agencies, "tbl_df")
})

test_that("tt_load loads data", {
  local_tt_master_file()
  local_tt_week_readme_html()
  local_tt_download_file_raw()
  expect_message(expect_message(expect_message(expect_message(expect_message(expect_message(
    {
      tt_data <- tt_load("2019-01-15")
    }
  ))))))
  expect_s3_class(tt_data, "tt_data")
  expect_length(tt_data, 2)
  expect_named(tt_data, c("agencies", "launches"))
})

test_that("print.tt_data lists the available datasets", {
  local_tt_master_file()
  local_tt_week_readme_html()
  local_tt_download_file_raw()
  local_readme()

  expect_message(expect_message(expect_message(expect_message(expect_message(
    expect_message({
      tt_data <- tt_load("2019-01-15")
    })
  )))))
  expect_snapshot({
    test_result <- print(tt_data)
  })

  expect_identical(test_result, tt_data)
})

test_that("Korean: Non-English encodings don't fail reading unicode from github", {
  skip_if_not(.Platform$OS.type == "windows")
  local_encoding("Korean")
  local_tt_master_file()
  local_tt_datasets()
  tt_data <- tt_datasets("2019")
  res <- nrow(data.frame(unclass(tt_data)))
  expect_equal(res, 52)
})

test_that("Japanese: Non-English encodings don't fail reading unicode from github", {
  skip_if_not(.Platform$OS.type == "windows")
  local_encoding("Japanese")
  local_tt_master_file()
  local_tt_datasets()
  tt_data <- tt_datasets("2019")
  res <- nrow(data.frame(unclass(tt_data)))
  expect_equal(res, 52)
})

test_that("Russian: Non-English encodings don't fail reading unicode from github", {
  skip_if_not(.Platform$OS.type == "windows")
  local_encoding("Russian")
  local_tt_master_file()
  local_tt_datasets()
  tt_data <- tt_datasets("2019")
  res <- nrow(data.frame(unclass(tt_data)))
  expect_equal(res, 52)
})

test_that("`tt_master_file()` updates the masterfile reference", {
  local_gh_get_sha_in_folder()
  local_gh_get_csv_data_type()
  ttmf <- tt_master_file()
  expect_true(is.data.frame(TT_MASTER_ENV$TT_MASTER_FILE))
  expect_true(nrow(TT_MASTER_ENV$TT_MASTER_FILE) > 0)
})

#' Raise a package-scoped error
#'
#' @inheritParams .shared-params
#' @inheritParams stbl::pkg_abort
#' @returns Does not return.
#' @keywords internal
.pkg_abort <- function(
  message,
  subclass,
  call = rlang::caller_env(),
  message_env = rlang::caller_env(),
  ...
) {
  stbl::pkg_abort(
    "tidytuesdayR",
    message,
    subclass,
    call = call,
    message_env = message_env,
    ...
  )
}

#' Set up a directory for dataset curation
#'
#' @inheritParams usethis::use_template
#' @inheritParams .shared-params
#'
#' @returns The resolved path (invisibly).
#' @keywords internal
prep_tt_curate <- function(
  path = "tt_submission",
  ignore = FALSE,
  env = rlang::caller_env()
) {
  rlang::check_installed("usethis", "to use curation templates.")
  rlang::check_installed("fs", "to use curation templates.")
  prep_proj(tt_submission_path = path, env = env)
  path <- usethis::proj_path(path)
  fs::dir_create(path)
  if (ignore) {
    usethis::use_build_ignore(path)
  }
  invisible(path)
}

prep_proj <- function(tt_submission_path, env = rlang::caller_env()) {
  rlang::check_installed("usethis", "to set up a temporary project.")
  tryCatch(
    usethis::proj_get(),
    # I can't easily test this, since the tests take place inside a project. At
    # most I can create a temporary project, but that's still a project.
    error = function(e) {
      # nocov start
      rlang::check_installed("fs", "to set up a temporary project.")
      proj_root <- fs::path_abs("..", tt_submission_path)
      here_path <- withr::local_file(
        fs::path(proj_root, ".here"),
        .local_envir = env
      )
      fs::file_create(here_path)
    } # nocov end
  )
}

test_that("prep_tt_curate creates the directory", {
  skip_on_ci()
  skip_on_cran()
  proj_dir <- withr::local_tempdir()
  usethis::local_project(proj_dir, force = TRUE, quiet = TRUE)
  full_submission_path <- usethis::proj_path("tt_submission")
  test_result <- prep_tt_curate()
  expect_equal(test_result, full_submission_path)
})

test_that("prep_tt_curate works with ignore", {
  skip_on_ci()
  skip_on_cran()
  proj_dir <- withr::local_tempdir()
  usethis::local_project(proj_dir, force = TRUE, quiet = TRUE)
  full_submission_path <- usethis::proj_path("tt_submission")
  test_result <- prep_tt_curate(ignore = TRUE)
  expect_equal(test_result, full_submission_path)
  ignore_path <- fs::path(proj_dir, ".Rbuildignore")
  expect_true(fs::file_exists(ignore_path))
  ignore_contents <- readLines(ignore_path)
  expect_true(grepl(full_submission_path, ignore_contents))
})

#' @title Listing all available TidyTuesdays
#'
#' @description
#' The TidyTuesday project is a constantly growing repository of data sets.
#' Knowing what type of data is available for each week requires going to the
#' source. However, one of the hallmarks of 'tidytuesdayR' is that you never
#' have to leave your R console. These functions were
#' created to help maintain this philosophy.
#'
#'
#' @details
#' To find out the available datasets for a specific year, the user
#' can use the function `tt_datasets()`. This function will either populate the
#' Viewer or print to console all the available data sets and the week/date
#' they are associated with.
#'
#' To get the whole list of all the data sets ever released by TidyTuesday, the
#' function `tt_available()` was created. This function will either populate the
#' Viewer or print to console all the available data sets ever made for
#' TidyTuesday.
#'
#' @name tt_available
#'
#' @examplesIf interactive()
#' # check to make sure there are requests still available
#' if (rate_limit_check(quiet = TRUE) > 30) {
#'   ## show data available from 2018
#'   tt_datasets(2018)
#'
#'   ## show all data available ever
#'   tt_available()
#' }
#'
NULL

#' @rdname tt_available
#' @inheritParams .shared-params
#' @export
#' @returns `tt_available()` returns a `tt_dataset_table_list`, which is a
#' list of `tt_dataset_table`. This class has special printing methods to show
#' the available data sets.
tt_available <- function(auth = gh::gh_token()) {
  tt_year <- sort(tt_years(auth = auth), decreasing = TRUE, )
  names(tt_year) <- tt_year
  datasets <- purrr::map(tt_year, ~ tt_datasets(.x, auth = auth))
  structure(datasets, class = c("tt_dataset_table_list"))
}

#' @rdname tt_available
#' @inheritParams .shared-params
#' @export
#' @returns `tt_datasets()` returns a `tt_dataset_table` object. This class has
#'  special printing methods to show the available datasets for the year.
tt_datasets <- function(year, auth = gh::gh_token()) {
  tt_check_year(year, auth = auth)

  ##############################################################################
  ## This portion changes if the dataset tables move, but it doesn't make sense
  ## to abstract it into a separate function.
  readme_html <- gh_get_readme_html(file.path("data", year), auth = auth)
  datasets <- rvest::html_table(readme_html)[[1]]
  ##############################################################################

  structure(datasets, .html = readme_html, class = "tt_dataset_table")
}

#' Printing Utilities for Listing Available Datasets
#'
#' printing utilities for showing the available datasets for a specific year or
#' all time
#' @inheritParams base::print
#' @param is_interactive Whether the function is being used interactively.
#' @returns `x`, invisibly
#' @name tt_print
#' @examplesIf interactive()
#' # check to make sure there are requests still available
#' if (rate_limit_check(quiet = TRUE) > 30) {
#'   available_datasets_2018 <- tt_datasets(2018)
#'   print(available_datasets_2018)
#'
#'   all_available_datasets <- tt_available()
#'   print(all_available_datasets)
#' }
NULL

#' @rdname tt_print
#' @export
print.tt_dataset_table <- function(x, ..., is_interactive = interactive()) {
  if (is_interactive) {
    tmpHTML <- save_tt_object(x, fn = make_tt_dataset_html)
    html_viewer(tmpHTML)
  } else {
    print(data.frame(unclass(x)))
  }
  invisible(x)
}

make_tt_dataset_html <- function(x, file = tempfile(fileext = ".html")) {
  readme <- attr(x, ".html")
  xml2::write_html(readme, file = file)
  invisible(readme)
}

save_tt_object <- function(x, fn) {
  tmpHTML <- tempfile(fileext = ".html")
  fn(x, file = tmpHTML)
  return(tmpHTML)
}

#' @rdname tt_print
#' @export
print.tt_dataset_table_list <- function(
  x,
  ...,
  is_interactive = interactive()
) {
  if (is_interactive) {
    tmpHTML <- save_tt_object(x, fn = make_tt_dataset_list_html)
    html_viewer(tmpHTML)
  } else {
    names(x) |>
      purrr::map(
        function(.x, x) {
          list(
            table = data.frame(unclass(x[[.x]])),
            year = .x
          )
        },
        x = x
      ) |>
      purrr::walk(
        function(.x) {
          cat(paste0("Year: ", .x$year, "\n\n"))
          print(.x$table)
          cat("\n\n")
        }
      )
  }
  invisible(x)
}

make_tt_dataset_list_html <- function(x, file = tempfile(fileext = ".html")) {
  readme <- names(x) |>
    purrr::map_chr(
      function(.x, x) {
        year_table <- attr(x[[.x]], ".html") |>
          rvest::html_element("table")
        paste(
          "<h2>",
          .x,
          "</h2>",
          as.character(year_table),
          ""
        )
      },
      x = x
    ) |>
    paste(collapse = "")

  readme <- paste(
    "<article class='markdown-body entry-content' itemprop='text'>",
    paste("<h1>TidyTuesday Datasets</h1>", readme),
    "</article>"
  ) |>
    xml2::read_html() |>
    github_page()

  xml2::write_html(readme, file = file)
  invisible(readme)
}

github_page <- function(page_content) {
  header <- paste0(
    "<head><link crossorigin=\"anonymous\" ",
    "media=\"all\" rel=\"stylesheet\" ",
    "href=\"https://cdnjs.cloudflare.com/ajax/libs/",
    "github-markdown-css/3.0.1/github-markdown.min.css\"></head>"
  )

  body <- page_content |>
    rvest::html_elements("body") |>
    as.character() |>
    enc2native()

  xml2::read_html(paste0(header, body))
}

test_that("tt_datasets throws errors when asking for invalid years", {
  local_tt_mocked_bindings(
    tt_years = function(...) {
      2018:2024
    }
  )
  stbl::expect_pkg_error_snapshot(
    tt_datasets(2017),
    "tidytuesdayR",
    "invalid_year"
  )
})

test_that("printing tt_datasets returns all the values as a printed data.frame if not interactive", {
  ds <- structure(
    data.frame(a = 1:2, b = 3:4),
    .html = "the_html",
    class = "tt_dataset_table"
  )
  expect_snapshot({
    print(ds, is_interactive = FALSE)
  })
})

test_that("tt_available returns object with all years data available", {
  local_tt_mocked_bindings(
    tt_years = function(...) {
      2019:2020
    },
    gh_get_readme_html = function(...) {
      xml2::read_html(test_path("fixtures", "readme2020.html"))
    }
  )
  ds <- tt_available()
  expect_s3_class(ds, "tt_dataset_table_list")
  expect_setequal(names(ds), as.character(tt_years()))

  ds_content <- as.list(unclass(ds))
  ds_content_data <-
    lapply(ds_content, function(x) {
      data.frame(unclass(x), stringsAsFactors = FALSE)
    })
  ds_content_html <-
    lapply(ds_content, function(x) {
      rvest::html_table(attr(x, ".html"))[[1]]
    })
  expect_equal(
    ds_content_html,
    ds_content_data,
    ignore_attr = TRUE
  )
})

test_that("printing tt_available returns all the values as a printed data.frame if not interactive", {
  ds <- structure(
    list(test_year = data.frame(a = 1:2, b = 3:4)),
    class = c("tt_dataset_table_list")
  )
  expect_snapshot({
    print(ds, is_interactive = FALSE)
  })
})

test_that("tt_dataset_table and tt_dataset_table_list objects can make html outputs", {
  local_tt_mocked_bindings(
    tt_years = function(...) {
      2019:2020
    },
    gh_get_readme_html = function(...) {
      xml2::read_html(test_path("fixtures", "readme2020.html"))
    }
  )
  ds_tl <- tt_available()
  ds_t <- tt_datasets(2019)

  tmpfile <- withr::local_tempfile(fileext = ".html")
  ds_tl_html <- make_tt_dataset_list_html(ds_tl, file = tmpfile)

  tmpfile2 <- withr::local_tempfile(fileext = ".html")
  ds_t_html <- make_tt_dataset_html(ds_t, file = tmpfile2)

  expect_true(file.exists(tmpfile))
  expect_true(file.exists(tmpfile2))
  expect_equal(
    xml2::read_html(tmpfile),
    ds_tl_html
  )
  expect_equal(
    xml2::read_html(tmpfile2),
    ds_t_html
  )
})

test_that("save_tt_object saves an html file", {
  expect_true(
    grepl("html$", {
      save_tt_object("", function(x, file) x)
    })
  )
})

test_that("tt_dataset_table and tt_dataset_table_list objects print through html_viewer when interactive", {
  local_mocked_bindings(
    html_viewer = function(url) {
      message(url)
    },
    save_tt_object = function(x, fn) {
      "tmpfile_path"
    }
  )
  ds <- structure("", class = "tt_dataset_table")
  expect_message(
    {
      print(ds, is_interactive = TRUE)
    },
    "tmpfile_path"
  )
  ds <- structure("", class = "tt_dataset_table_list")
  expect_message(
    {
      print(ds, is_interactive = TRUE)
    },
    "tmpfile_path"
  )
})

library(tidyverse)
library(janitor)

states <- state.x77 |>
  tibble::as_tibble(rownames = "state") |>
  janitor::clean_names() |>
  dplyr::mutate(
    dplyr::across(
      c("population", "income", "frost", "area"),
      as.integer
    )
  )

#' Save datasets for submission
#'
#' Datasets for TidyTuesday submissions should be saved in a specific format,
#' with an accompanying data dictionary `dataset_name.md` file. This function
#' saves the dataset as a CSV file in your submission directory (creating the
#' submission directory if it does not already exist), and creates a data
#' dictionary file for you to fill out. If you're in an interactive session, the
#' dictionary file is opened for editing.
#'
#' @inheritParams usethis::use_template
#' @inheritParams .shared-params
#' @param dataset The clean dataset to save. The dataset must be a data.frame.
#' @param dataset_name The name to save the dataset as. By default, the name of
#'   the dataset variable is used.
#'
#' @returns A logical vector indicating whether the file was created or
#'   modified, invisibly.
#' @export
#'
#' @examplesIf interactive()
#'
#'   tt_save_dataset(mtcars)
tt_save_dataset <- function(
  dataset,
  path = "tt_submission",
  dataset_name = rlang::caller_arg(dataset),
  open = rlang::is_interactive(),
  ignore = FALSE
) {
  prep_tt_curate(path, ignore = ignore)
  rlang::check_installed(c("readr"), "to save the dataset CSV.")
  path <- usethis::proj_path(path)

  dataset_path <- fs::path(path, dataset_name, ext = "csv")
  readr::write_csv(dataset, dataset_path)

  dictionary <- create_tt_dict(dataset)
  dictionary_path <- fs::path(path, dataset_name, ext = "md")
  cat(dictionary, file = dictionary_path, sep = "\n")
  usethis::edit_file(dictionary_path, open = open)
  return(invisible(TRUE))
}

create_tt_dict <- function(x) {
  rlang::check_installed(
    c("tibble", "dplyr", "knitr", "vctrs"),
    "to create a data dictionary."
  )
  tibble::tibble(variable = names(x)) |>
    dplyr::mutate(
      class = purrr::map(x, \(var) vctrs::vec_ptype_full(var)),
      description = "Describe this field in sentence case."
    ) |>
    knitr::kable()
}

test_that("tt_save_dataset creates the expected files", {
  skip_on_ci()
  skip_on_cran()
  proj_dir <- withr::local_tempdir()
  full_submission_path <- fs::path(proj_dir, "tt_submission")
  usethis::local_project(proj_dir, force = TRUE, quiet = TRUE)
  expect_true(tt_save_dataset(mtcars, open = FALSE))
  file_path <- fs::path(full_submission_path, "mtcars.csv")
  expect_snapshot_file(file_path)
  file_path <- fs::path(full_submission_path, "mtcars.md")
  expect_snapshot_file(file_path)
})

#' Submit a TidyTuesday dataset
#'
#' Submit a curated dataset for review by uploading it to GitHub and creating a
#' pull request. The dataset should be prepared using [tt_clean()],
#' [tt_save_dataset()], [tt_intro()], and [tt_meta()]. You can also use this
#' function to submit changes to your local copies of the files.
#'
#' @inheritParams .shared-params
#' @param open Whether to open the pull request in a browser. Defaults to `TRUE`
#'   in an interactive session.
#'
#' @returns The URL of the pull request, invisibly.
#' @export
#'
#' @examplesIf interactive()
#' # First set up a dataset in the "tt_submission" folder.
#' tt_submit()
tt_submit <- function(
  path = "tt_submission",
  auth = gh::gh_token(),
  open = rlang::is_interactive()
) {
  rlang::check_installed("base64enc", "to prepare files for a submission.")

  files <- tt_find_dataset_files(path)

  auth <- gh_auth_check(auth)
  user <- tt_user(auth = auth)
  repo <- getOption("tidytuesdayR.tt_repo", "rfordatascience/tidytuesday")
  branch <- tt_find_branch(path)

  fork_info <- tt_fork(user = user, repo = repo, auth = auth)
  tt_branch_create(fork_info = fork_info, branch = branch, auth = auth)
  tt_branch_populate(
    fork_repo = fork_info$full_name,
    branch = branch,
    files = files,
    auth = auth
  )

  existing_prs <- call_gh(
    "/repos/{repo}/pulls",
    head = glue::glue("{user}:{branch}"),
    repo = repo,
    state = "open",
    auth = auth
  )

  if (length(existing_prs)) {
    # I tested this manually, leaving it at that for now.
    pr_url <- existing_prs[[1]]$html_url # nocov
    cli::cli_inform("View PR at {.url {pr_url}}") # nocov
  } else {
    pr_url <- glue::glue(
      "https://github.com/{repo}/compare/main...{user}:{branch}"
    )
    cli::cli_inform("Create PR at {.url {pr_url}}")
  }

  if (open) {
    utils::browseURL(pr_url) # nocov
  }
  return(invisible(pr_url))
}

tt_find_dataset_files <- function(path = "tt_submission") {
  files <- list.files(path, full.names = TRUE)
  files <- setdiff(files, fs::path(path, "branch.txt"))
  expected_files <- tt_find_expected_files(path)
  csv_files <- tt_find_csv_files(path)
  dictionary_files <- tt_find_dictionaries(csv_files)
  img_files <- tt_find_images(path)
  known_files <- c(expected_files, csv_files, dictionary_files, img_files)
  extra_files <- setdiff(files, known_files)
  if (length(extra_files)) {
    .pkg_abort(
      c(
        "{.arg path} should only contain submission files.",
        x = "Extra files: {extra_files}"
      ),
      "extra_files"
    )
  }
  return(known_files)
}

tt_find_expected_files <- function(path = "tt_submission") {
  expected_files <- fs::path(
    path,
    c("cleaning.R", "intro.md", "meta.yaml")
  )
  missing_files <- expected_files[!fs::file_exists(expected_files)]
  if (length(missing_files)) {
    .pkg_abort(
      c(
        "All expected files must exist in {.arg path}.",
        x = "Missing files: {missing_files}"
      ),
      "missing_expected"
    )
  }
  return(expected_files)
}

tt_find_csv_files <- function(path = "tt_submission") {
  csv_files <- unname(fs::dir_ls(path, glob = "*.csv"))
  return(tt_validate_csv_sizes(csv_files))
}

tt_validate_csv_sizes <- function(csv_files) {
  max_size <- fs::fs_bytes("25MB")
  file_sizes <- fs::file_size(csv_files)
  too_large <- file_sizes > max_size
  if (any(too_large)) {
    large_files <- csv_files[too_large]
    large_sizes <- file_sizes[too_large]
    .pkg_abort(
      c(
        "CSV files must be <= 25MB to upload to GitHub.",
        x = "Files too large:",
        rlang::set_names(
          purrr::map2_chr(
            large_files,
            large_sizes,
            \(file, size) paste0(file, " (", format(size), ")")
          ),
          rep("*", length(large_files))
        )
      ),
      "csv_size"
    )
  }
  return(csv_files)
}

tt_find_images <- function(path = "tt_submission") {
  rlang::check_installed("yaml", "to verify image files.")
  yaml_path <- fs::path(path, "meta.yaml")
  if (fs::file_exists(yaml_path)) {
    meta <- yaml::read_yaml(yaml_path)
    if (length(meta$images)) {
      expected_images <- fs::path(path, purrr::map_chr(meta$images, "file"))
      missing_files <- expected_images[!fs::file_exists(expected_images)]
      if (length(missing_files)) {
        .pkg_abort(
          c(
            "All images in meta.yaml must exist in {.arg path}.",
            x = "Missing images: {missing_files}"
          ),
          "missing_images"
        )
      }
      # Check and resize images if needed
      for (image in meta$images) {
        tt_check_and_resize_image_single(image, path)
      }
      return(expected_images)
    }
  }
  .pkg_abort(
    "No images found in meta.yaml",
    "no_images"
  )
}

tt_check_and_resize_image_single <- function(image, path) {
  max_bsky_size <- fs::fs_bytes("976.56KB")
  img_path <- fs::path(path, image$file)
  img_size <- fs::file_size(img_path)

  if (img_size > max_bsky_size) {
    rlang::check_installed("magick", "to resize images.")
    tt_inform_image_resize(image$file, img_size, max_bsky_size)
    ratio <- tt_calculate_resize_ratio(img_size, max_bsky_size)
    resized_img <- tt_resize_image(img_path, ratio)
    tt_confirm_resized_image(resized_img, img_path, ratio)
    magick::image_write(resized_img, img_path)
    new_size <- fs::file_size(img_path)
    tt_inform_resize_complete(img_size, new_size)
  }

  return(invisible(NULL))
}

tt_inform_image_resize <- function(filename, current_size, max_size) {
  cli::cli_inform(c(
    "i" = "Image {.file {filename}} is {format(current_size)}, which exceeds the Bluesky limit of {format(max_size)}.",
    "i" = "Resizing image to fit within the limit..."
  ))
}

COMPRESSION_SAFETY_FACTOR <- 90L

tt_calculate_resize_ratio <- function(img_size, max_size) {
  # Round down to make sure we're *under* 1MB. This isn't actually guaranteed to
  # work because image size isn't directly proportional to file size, but it
  # errs on the side of making things smaller than they need to be.
  floor(as.integer(max_size) / as.integer(img_size) * COMPRESSION_SAFETY_FACTOR)
}

tt_resize_image <- function(img_path, ratio) {
  magick::image_read(img_path) |>
    magick::image_resize(magick::geometry_size_percent(ratio))
}

tt_confirm_resized_image <- function(resized_img, img_path, ratio) {
  if (!rlang::is_interactive() || is.null(getOption("viewer"))) {
    return(invisible(NULL))
  }

  temp_preview <- withr::local_tempfile(fileext = fs::path_ext(img_path))
  magick::image_write(resized_img, temp_preview)
  viewer <- getOption("viewer")
  viewer(temp_preview)

  response <- utils::menu(
    choices = c("Yes, use resized image", "No, cancel submission"),
    title = sprintf(
      "Image resized to %d%% of original. Does it look acceptable?",
      ratio
    )
  )

  if (response != 1) {
    .pkg_abort("Submission cancelled by user.", "cancelled")
  }

  return(invisible(NULL))
}

tt_inform_resize_complete <- function(original_size, new_size) {
  cli::cli_inform(c(
    "v" = "Image resized from {format(original_size)} to {format(new_size)}."
  ))
}

tt_find_dictionaries <- function(csv_files) {
  expected_md_files <- fs::path_ext_set(csv_files, "md")
  missing_files <- expected_md_files[!fs::file_exists(expected_md_files)]
  if (length(missing_files)) {
    .pkg_abort(
      c(
        "All datasets must have an associated md file in {.arg path}.",
        x = "Missing dictionaries: {missing_files}"
      ),
      "missing_dictionaries"
    )
  }
  return(expected_md_files)
}

tt_find_branch <- function(path = "tt_submission") {
  branch_tag_path <- fs::path(path, "branch.txt")
  if (fs::file_exists(branch_tag_path)) {
    return(stringr::str_trim(readLines(branch_tag_path)))
  }
  branch <- unclass(
    glue::glue("submission-{today()}-{round(runif(1)*100000)}")
  )
  writeLines(branch, branch_tag_path)
  return(branch)
}

# Skipping coverage of the gh stuff for now.
# nocov start
call_gh <- function(..., auth = gh::gh_token()) {
  gh::gh(..., .token = auth)
}
# nocov end

tt_user <- function(auth = gh::gh_token()) {
  call_gh("GET /user", auth = auth)$login
}

tt_fork <- function(user, repo, auth = gh::gh_token()) {
  # GitHub automatically returns existing fork if it exists, no need to check
  my_fork <- call_gh("POST /repos/{repo}/forks", repo = repo, auth = auth)
  tt_sync_fork(
    fork_repo = my_fork$full_name,
    branch = my_fork$default_branch,
    auth = auth
  )
  return(my_fork)
}

tt_sync_fork <- function(fork_repo, branch, auth = gh::gh_token()) {
  call_gh(
    "POST /repos/{fork_repo}/merge-upstream",
    fork_repo = fork_repo,
    branch = branch,
    auth = auth
  )
  return(invisible(NULL))
}

tt_branch_create <- function(fork_info, branch, auth = gh::gh_token()) {
  fork_repo <- fork_info$full_name
  default_branch <- fork_info$default_branch

  main_sha <- call_gh(
    "GET /repos/{fork_repo}/git/refs/heads/{default_branch}",
    fork_repo = fork_repo,
    default_branch = default_branch,
    auth = auth
  )$object$sha
  target_ref <- glue::glue("refs/heads/{branch}")
  existing <- call_gh(
    "GET /repos/{fork_repo}/git/refs/heads",
    fork_repo = fork_repo,
    auth = auth
  )
  if (!target_ref %in% purrr::map_chr(existing, "ref")) {
    call_gh(
      "POST /repos/{fork_repo}/git/refs",
      fork_repo = fork_repo,
      ref = glue::glue("refs/heads/{branch}"),
      sha = main_sha,
      auth = auth
    )
  }
  return(invisible(target_ref))
}

tt_branch_populate <- function(
  fork_repo,
  branch,
  files,
  auth = gh::gh_token()
) {
  existing_content <- tt_branch_content(fork_repo, branch, auth)
  purrr::walk(files, function(file) {
    content <- base64enc::base64encode(file)
    filename <- basename(file)
    sha <- existing_content[[filename]]
    if (!identical(sha, git_blob_sha1(file))) {
      action <- ifelse(is.null(sha), "Add", "Update")
      call_gh(
        "PUT /repos/{fork_repo}/contents/data/curated/new_submission/{filename}",
        fork_repo = fork_repo,
        filename = filename,
        message = glue::glue("{action} {filename}"),
        content = content,
        branch = branch,
        sha = sha,
        auth = auth
      )
    }
  })
}

tt_branch_content <- function(fork_repo, branch, auth = gh::gh_token()) {
  # If the branch has a "new_submission" folder, return the contents of that
  # folder.
  new_submission <- tryCatch(
    call_gh(
      "GET /repos/{fork_repo}/contents/data/curated/new_submission",
      fork_repo = fork_repo,
      ref = glue::glue("refs/heads/{branch}"),
      auth = auth
    ),
    error = function(e) {
      NULL
    }
  )
  existing_file_names <- purrr::map_chr(new_submission, "name")
  existing_file_shas <- purrr::map(new_submission, "sha")
  return(rlang::set_names(existing_file_shas, existing_file_names))
}

git_blob_sha1 <- function(file) {
  rlang::check_installed("openssl", "to check for file changes.")
  size <- file.info(file)$size
  contents <- readBin(file, "raw", size)
  unclass(
    as.character(
      openssl::sha1(
        c(
          charToRaw(sprintf("blob %d", size)),
          as.raw(0),
          contents
        )
      )
    )
  )
}

test_that("tt_find_dataset_files finds expected files", {
  expect_identical(
    tt_find_dataset_files(test_path("fixtures", "tt_submission")),
    c(
      test_path("fixtures", "tt_submission", "cleaning.R"),
      test_path("fixtures", "tt_submission", "intro.md"),
      test_path("fixtures", "tt_submission", "meta.yaml"),
      test_path("fixtures", "tt_submission", "states.csv"),
      test_path("fixtures", "tt_submission", "states.md"),
      test_path("fixtures", "tt_submission", "states_population.png")
    )
  )
})

test_that("tt_find_dataset_files errors informatively for extra files", {
  stbl::expect_pkg_error_snapshot(
    tt_find_dataset_files(test_path("fixtures", "tt_submission_extra")),
    "tidytuesdayR",
    "extra_files"
  )
})

test_that("tt_find_dataset_files errors informatively for missing files", {
  stbl::expect_pkg_error_snapshot(
    tt_find_dataset_files(test_path("fixtures", "tt_submission_missing")),
    "tidytuesdayR",
    "missing_expected"
  )
})

test_that("tt_find_dataset_files errors informatively for missing images", {
  stbl::expect_pkg_error_snapshot(
    tt_find_dataset_files(test_path(
      "fixtures",
      "tt_submission_missing_image1"
    )),
    "tidytuesdayR",
    "missing_images"
  )
  stbl::expect_pkg_error_snapshot(
    tt_find_dataset_files(test_path(
      "fixtures",
      "tt_submission_missing_image2"
    )),
    "tidytuesdayR",
    "no_images"
  )
})

test_that("tt_find_dataset_files errors informatively for missing dictionary", {
  stbl::expect_pkg_error_snapshot(
    tt_find_dataset_files(test_path("fixtures", "tt_submission_missing_md")),
    "tidytuesdayR",
    "missing_dictionaries"
  )
})

test_that("tt_find_branch deals with branch names", {
  path <- withr::local_tempdir()
  branch <- tt_find_branch(path)
  expect_match(
    branch,
    "submission-\\d{4}-\\d{2}-\\d{2}",
    all = TRUE
  )
  branch_file_path <- fs::path(path, "branch.txt")
  expect_true(fs::file_exists(branch_file_path))
  branch2 <- tt_find_branch(path)
  expect_identical(branch2, branch)
})

test_that("tt_user extracts login from GitHub response", {
  local_mocked_call_gh(function(endpoint, ..., auth) {
    expect_equal(endpoint, "GET /user")
    expect_equal(auth, "test_token")
    return(list(login = "testuser", id = 12345))
  })

  result <- tt_user(auth = "test_token")
  expect_equal(result, "testuser")
})

test_that("tt_fork returns fork info from GitHub and syncs it (#147)", {
  local_mocked_bindings(
    tt_sync_fork = function(fork_repo, branch, auth) {
      expect_equal(fork_repo, "testuser/tidytuesday")
      expect_equal(branch, "main")
      expect_equal(auth, "test_token")
      return(invisible(NULL))
    }
  )
  local_mocked_call_gh(function(endpoint, repo, auth, ...) {
    expect_equal(endpoint, "POST /repos/{repo}/forks")
    expect_equal(repo, "rfordatascience/tidytuesday")
    expect_equal(auth, "test_token")
    return(list(
      full_name = "testuser/tidytuesday",
      default_branch = "main",
      owner = list(login = "testuser")
    ))
  })

  result <- tt_fork(
    user = "testuser",
    repo = "rfordatascience/tidytuesday",
    auth = "test_token"
  )
  expect_equal(result$full_name, "testuser/tidytuesday")
  expect_equal(result$default_branch, "main")
})

test_that("tt_sync_fork calls merge-upstream API (#147)", {
  local_mocked_call_gh(function(endpoint, fork_repo, branch, auth, ...) {
    expect_equal(endpoint, "POST /repos/{fork_repo}/merge-upstream")
    expect_equal(fork_repo, "testuser/tidytuesday")
    expect_equal(branch, "main")
    expect_equal(auth, "test_token")
    return(list(
      message = "Successfully fetched and fast-forwarded from upstream",
      merge_type = "fast-forward",
      base_branch = "upstream:main"
    ))
  })

  expect_invisible(
    tt_sync_fork(
      fork_repo = "testuser/tidytuesday",
      branch = "main",
      auth = "test_token"
    )
  )
})

test_that("tt_branch_create creates new branch from default branch", {
  local_mocked_call_gh(function(
    endpoint,
    fork_repo,
    default_branch,
    auth,
    ref,
    sha,
    ...
  ) {
    if (endpoint == "GET /repos/{fork_repo}/git/refs/heads/{default_branch}") {
      expect_equal(fork_repo, "testuser/tidytuesday")
      expect_equal(default_branch, "main")
      return(list(object = list(sha = "abc123")))
    }
    if (endpoint == "GET /repos/{fork_repo}/git/refs/heads") {
      expect_equal(fork_repo, "testuser/tidytuesday")
      return(list()) # No existing branches
    }
    if (endpoint == "POST /repos/{fork_repo}/git/refs") {
      expect_equal(fork_repo, "testuser/tidytuesday")
      expect_equal(ref, "refs/heads/test-branch")
      expect_equal(sha, "abc123")
      return(list(ref = "refs/heads/test-branch"))
    }
  })

  fork_info <- list(full_name = "testuser/tidytuesday", default_branch = "main")
  result <- tt_branch_create(fork_info, "test-branch", auth = "test_token")
  expect_equal(result, "refs/heads/test-branch")
})

test_that("tt_branch_create skips if branch already exists", {
  local_mocked_call_gh(function(
    endpoint,
    fork_repo,
    default_branch,
    auth,
    ...
  ) {
    if (endpoint == "GET /repos/{fork_repo}/git/refs/heads/{default_branch}") {
      return(list(object = list(sha = "abc123")))
    }
    if (endpoint == "GET /repos/{fork_repo}/git/refs/heads") {
      return(list(
        list(ref = "refs/heads/main"),
        list(ref = "refs/heads/test-branch") # Branch already exists
      ))
    }
    stop("Should not create branch if it already exists")
  })

  fork_info <- list(full_name = "testuser/tidytuesday", default_branch = "main")
  result <- tt_branch_create(fork_info, "test-branch", auth = "test_token")
  expect_equal(result, "refs/heads/test-branch")
})

test_that("tt_branch_content returns file info from GitHub", {
  local_mocked_call_gh(function(endpoint, fork_repo, ref, auth, ...) {
    expect_equal(
      endpoint,
      "GET /repos/{fork_repo}/contents/data/curated/new_submission"
    )
    expect_equal(fork_repo, "testuser/tidytuesday")
    expect_equal(ref, "refs/heads/test-branch")
    return(list(
      list(name = "data.csv", sha = "sha123"),
      list(name = "meta.yaml", sha = "sha456")
    ))
  })

  result <- tt_branch_content(
    "testuser/tidytuesday",
    "test-branch",
    auth = "test_token"
  )
  expect_equal(names(result), c("data.csv", "meta.yaml"))
  expect_equal(result[["data.csv"]], "sha123")
  expect_equal(result[["meta.yaml"]], "sha456")
})

test_that("tt_branch_content returns empty list if folder doesn't exist", {
  local_mocked_call_gh(function(endpoint, fork_repo, ref, auth, ...) {
    stop("404 error")
  })

  result <- tt_branch_content(
    "testuser/tidytuesday",
    "test-branch",
    auth = "test_token"
  )
  expect_equal(result, structure(list(), names = character(0)))
})

test_that("git_blob_sha1 calculates git blob SHA correctly", {
  # Create a temporary test file
  temp_file <- withr::local_tempfile()
  writeLines("test content", temp_file)

  result <- git_blob_sha1(temp_file)

  # Should return a 40-character hex string (SHA-1 hash)
  expect_type(result, "character")
  expect_match(result, "^[a-f0-9]{40}$")

  # Same file should produce same hash
  result2 <- git_blob_sha1(temp_file)
  expect_equal(result, result2)
})

test_that("tt_branch_populate adds new file when it doesn't exist", {
  temp_file <- withr::local_tempfile()
  writeLines("test content", temp_file)

  local_mocked_bindings(
    tt_branch_content = function(...) {
      return(structure(list(), names = character(0))) # No existing files
    }
  )
  local_mocked_call_gh(function(
    endpoint,
    fork_repo,
    filename,
    message,
    content,
    branch,
    sha,
    auth,
    ...
  ) {
    expect_equal(
      endpoint,
      "PUT /repos/{fork_repo}/contents/data/curated/new_submission/{filename}"
    )
    expect_equal(fork_repo, "testuser/tidytuesday")
    expect_match(message, "^Add ")
    expect_null(sha)
    return(list(content = list(name = filename)))
  })

  expect_no_error(
    tt_branch_populate(
      fork_repo = "testuser/tidytuesday",
      branch = "test-branch",
      files = temp_file,
      auth = "test_token"
    )
  )
})

test_that("tt_branch_populate updates existing file when SHA differs", {
  temp_file <- withr::local_tempfile()
  writeLines("new content", temp_file)

  local_mocked_bindings(
    tt_branch_content = function(...) {
      return(structure(
        list("different_sha"),
        names = basename(temp_file)
      ))
    }
  )
  local_mocked_call_gh(function(
    endpoint,
    fork_repo,
    filename,
    message,
    content,
    branch,
    sha,
    auth,
    ...
  ) {
    expect_equal(
      endpoint,
      "PUT /repos/{fork_repo}/contents/data/curated/new_submission/{filename}"
    )
    expect_match(message, "^Update ")
    expect_equal(sha, "different_sha")
    return(list(content = list(name = filename)))
  })

  expect_no_error(
    tt_branch_populate(
      fork_repo = "testuser/tidytuesday",
      branch = "test-branch",
      files = temp_file,
      auth = "test_token"
    )
  )
})

test_that("tt_branch_populate skips file when SHA matches", {
  temp_file <- withr::local_tempfile()
  writeLines("test content", temp_file)

  # Get the actual SHA for this file
  file_sha <- git_blob_sha1(temp_file)

  local_mocked_bindings(
    tt_branch_content = function(...) {
      return(structure(
        list(file_sha),
        names = basename(temp_file)
      ))
    }
  )
  local_mocked_call_gh(function(...) {
    stop("Should not be called when SHA matches")
  })

  expect_no_error(
    tt_branch_populate(
      fork_repo = "testuser/tidytuesday",
      branch = "test-branch",
      files = temp_file,
      auth = "test_token"
    )
  )
})

test_that("tt_submit informs about the PR url", {
  local_mocked_bindings(
    gh_auth_check = function(...) {
      return("auth")
    },
    tt_user = function(auth) {
      return("testuser")
    },
    tt_fork = function(...) {
      return(list(full_name = "testuser/tidytuesday", default_branch = "main"))
    },
    tt_find_branch = function(...) {
      return("submission-2025-01-01")
    },
    tt_branch_create = function(...) {
      return("branch_response")
    },
    tt_branch_populate = function(...) {
      return("populate_response")
    }
  )
  local_mocked_call_gh(function(endpoint, ...) {
    if (grepl("/pulls", endpoint)) {
      return(list()) # No existing PRs
    }
    stop("Unexpected call_gh endpoint: ", endpoint)
  })
  expect_message(
    {
      test_result <- expect_invisible({
        tt_submit(
          test_path("fixtures", "tt_submission"),
          auth = "noauth",
          open = FALSE
        )
      })
    },
    "Create PR at"
  )
  expect_identical(
    test_result,
    glue::glue(
      "https://github.com/rfordatascience/tidytuesday/compare/main...testuser:submission-2025-01-01"
    )
  )
})

# CSV file validation --------------------------------------------------------

test_that("tt_find_csv_files passes when CSVs are under 25MB", {
  expect_no_error(
    tt_find_csv_files(test_path("fixtures", "tt_submission"))
  )
})

test_that("tt_find_csv_files errors when CSV exceeds 25MB", {
  local_mocked_bindings(
    file_size = function(path) {
      fs::fs_bytes("30MB")
    },
    .package = "fs"
  )
  stbl::expect_pkg_error_snapshot(
    tt_find_csv_files(test_path("fixtures", "tt_submission")),
    "tidytuesdayR",
    "csv_size"
  )
})

test_that("tt_find_csv_files handles multiple large CSVs", {
  local_mocked_bindings(
    dir_ls = function(path, glob) {
      c(
        test_path("fixtures", "tt_submission", "file1.csv"),
        test_path("fixtures", "tt_submission", "file2.csv")
      )
    },
    file_size = function(path) {
      c(fs::fs_bytes("30MB"), fs::fs_bytes("40MB"))
    },
    .package = "fs"
  )

  stbl::expect_pkg_error_snapshot(
    tt_find_csv_files(test_path("fixtures", "tt_submission")),
    "tidytuesdayR",
    "csv_size"
  )
})

# Image validation and resizing ----------------------------------------------

test_that("tt_find_images passes when images are under limit", {
  expect_no_error(
    tt_find_images(test_path("fixtures", "tt_submission"))
  )
})

test_that("tt_find_images resizes large images non-interactively", {
  # Use temp dir so we can resize.
  temp_dir <- withr::local_tempdir()
  fs::file_copy(
    fs::dir_ls(test_path("fixtures", "tt_submission_large_img")),
    temp_dir
  )
  img_path <- fs::path(temp_dir, "states_population.png")
  withr::local_options(list(rlang_interactive = FALSE))
  expect_message(
    expect_message(
      tt_find_images(temp_dir),
      "exceeds the Bluesky limit"
    ),
    "Image resized from"
  )
  expect_lte(
    unclass(fs::file_size(img_path)),
    unclass(fs::fs_bytes("976.56KB"))
  )
})

test_that("tt_find_images cancels on user rejection", {
  withr::local_options(
    list(
      rlang_interactive = TRUE,
      viewer = function(x) invisible(NULL)
    )
  )
  local_mocked_bindings(
    menu = function(choices, title) {
      2 # Choose "No, cancel submission"
    },
    .package = "utils"
  )
  expect_message(
    stbl::expect_pkg_error_classes(
      tt_find_images(
        test_path("fixtures", "tt_submission_large_img")
      ),
      "tidytuesdayR",
      "cancelled"
    ),
    "exceeds the Bluesky limit"
  )
})

test_that("tt_find_images accepts on user approval", {
  # Use temp dir so we can resize.
  temp_dir <- withr::local_tempdir()
  fs::file_copy(
    fs::dir_ls(test_path("fixtures", "tt_submission_large_img")),
    temp_dir
  )
  withr::local_options(
    list(
      rlang_interactive = TRUE,
      viewer = function(x) invisible(NULL)
    )
  )
  local_mocked_bindings(
    menu = function(choices, title) {
      1 # Choose "Yes, use resized image"
    },
    .package = "utils"
  )
  expect_message(
    expect_message(
      tt_find_images(temp_dir),
      "exceeds the Bluesky limit"
    ),
    "Image resized from"
  )
})

#' Guidance for TidyTuesday dataset curation
#'
#' Open an R script to guide you through the process of curating and submitting
#' a TidyTuesday dataset. See `vignette("curating", package = "tidytuesdayR)`
#' for more information.
#'
#' @inheritParams usethis::edit_file
#'
#' @returns The path to the `tt_curation.R` script, invisibly.
#' @export
#'
#' @examples
#' tt_curate_data()
tt_curate_data <- function(open = rlang::is_interactive()) {
  path <- system.file("templates", "tt_curation.R", package = "tidytuesdayR")
  if (open) {
    # nocov start
    rlang::check_installed("usethis", "to open the curation script.")
    usethis::edit_file(path)
    # nocov end
  }
  return(invisible(path))
}

test_that("tt_curate_data returns the path to the curation script", {
  rlang::local_interactive(FALSE)
  test_result <- tt_curate_data()
  expect_equal(
    test_result,
    system.file("templates", "tt_curation.R", package = "tidytuesdayR")
  )
})

test_that("tt_curate_data respects open parameter", {
  # When open = FALSE, should not attempt to open file
  test_result <- tt_curate_data(open = FALSE)
  expect_equal(
    test_result,
    system.file("templates", "tt_curation.R", package = "tidytuesdayR")
  )
})

#' Find the most recent tuesday
#'
#' Identify the most recent 'TidyTuesday' date relative to a specified date.
#'
#' @param date A date as a date object or character string in `YYYY-MM-DD`
#'   format. Defaults to today's date.
#'
#' @returns The TidyTuesday date in the same week as the specified date, using
#'   Monday as the start of the week.
#'
#' @examples
#' last_tuesday() # get last Tuesday relative to today's date
#' last_tuesday("2020-01-01") # get last Tuesday relative to a specified date
#'
#' @export
last_tuesday <- function(date = today(tzone = "America/New_York")) {
  if (!valid_date(date)) {
    .pkg_abort(
      "{.val {date}} cannot be coerced to a {.cls Date}.",
      "invalid_date"
    )
  }
  date <- as.Date(tt_date_format(date))
  diff_tuesday <- subtract_tuesday(date)
  tuesday <- date + diff_tuesday

  # data is usually released on a monday
  #
  # TODO: Don't do this logic here, do it where this is called when that makes
  # sense.
  if (diff_tuesday > 1) {
    tuesday <- tuesday - 7
  }

  return(tuesday)
}

subtract_tuesday <- function(date) {
  diff_tuesday <- 3 - lubridate::wday(date, week_start = 7)

  if (diff_tuesday < 0) {
    diff_tuesday <- diff_tuesday + 7
  }
  diff_tuesday
}

test_that("last_tuesday will give you the most recent tuesday", {
  ## Look backwards to the last tt
  date_1 <- as.Date("2020-01-01")
  last_tuesday_1 <- last_tuesday(date_1)

  ## Look forwards to the "next" tt (they can be posted on mondays)
  date_2 <- as.Date("2020-01-06")
  last_tuesday_2 <- last_tuesday(date_2)

  ## day of returns same day
  date_3 <- as.Date("2020-01-07")
  last_tuesday_3 <- last_tuesday(date_2)

  expect_equal(
    last_tuesday_1,
    as.Date("2019-12-31")
  )
  expect_equal(
    last_tuesday_2,
    as.Date("2020-01-07")
  )
  expect_equal(
    last_tuesday_3,
    as.Date("2020-01-07")
  )
})

test_that("last_tuesday errors with bad dates", {
  stbl::expect_pkg_error_snapshot(
    last_tuesday("blue"),
    "tidytuesdayR",
    "invalid_date"
  )
  stbl::expect_pkg_error_snapshot(
    last_tuesday(1),
    "tidytuesdayR",
    "invalid_date"
  )
})

#' Download TidyTuesday data
#'
#' Download all or specific files identified in a TidyTuesday dataset.
#'
#' @inheritParams .shared-params
#' @inheritParams tt_download_file
#'
#' @returns A list of tibbles from the downloaded files.
#'
#' @export
#' @examplesIf interactive()
#' # Get the list of files for a week.
#' tt_output <- tt_load_gh("2019-01-15")
#'
#' # Download a specific file.
#' agencies <- tt_download(tt_output, files = "agencies.csv")
tt_download <- function(tt, files = "All", ..., auth = gh::gh_token()) {
  files <- tt_check_files(tt, files)
  cli::cli_h1("Downloading files")
  files_len <- length(files)
  tt_data <- purrr::imap(files, function(file, file_n) {
    cli::cli_inform(
      c(" " = "{file_n} of {files_len}: {.val {file}}"),
      class = "tt-message-download"
    )
    tt_download_file(tt, file, ..., auth = auth)
  })
  names(tt_data) <- tools::file_path_sans_ext(files)
  return(tt_data)
}

tt_check_files <- function(tt, files, call = rlang::caller_env()) {
  all_files <- attr(tt, ".files")$data_files
  files <- rlang::try_fetch(
    {
      files <- match.arg(
        files,
        choices = c("All", all_files),
        several.ok = TRUE
      )
    },
    error = function(cnd) {
      .pkg_abort(
        "{.arg files} must be one or more of {.or {.val {all_files}}}, or {.val All}.",
        "bad_file",
        call = call
      )
    }
  )

  if ("All" %in% files) {
    files <- all_files
  }
  return(files)
}

# Hit the api ----

# This one is purely a wrapper around gh::gh(), so at least for now we'll trust
# that it works.
#
# nocov start

#' Get data from the tt github repo.
#'
#' @inheritParams .shared-params
#' @param path Path within the `rfordatascience/tidytuesday` repo.
#' @param ... Additional parameters passed to [gh::gh()].
#'
#' @returns The GitHub response as parsed by [gh::gh()].
#' @keywords internal
gh_get <- function(path, auth = gh::gh_token(), ...) {
  auth <- gh_auth_check(auth)
  gh::gh(
    "/repos/:tt_repo/contents/:path",
    path = path,
    tt_repo = getOption("tidytuesdayR.tt_repo"),
    .token = auth,
    ...
  )
}

gh_get_url <- function(path, auth = gh::gh_token(), ...) {
  gh::gh(path, .token = auth, ...)
}
# nocov end

gh_get_sha_in_folder <- function(path, file, auth = gh::gh_token()) {
  folder <- gh_get(path, auth = auth)
  return(gh_extract_sha_in_folder(folder, file))
}

gh_get_csv <- function(path, header = TRUE, auth = gh::gh_token()) {
  gh_response <- gh_get(path, auth = auth)
  return(gh_extract_csv(gh_response, header = header))
}

gh_get_folder <- function(path, auth = gh::gh_token()) {
  gh_response <- gh_get(path, auth = auth)
  tidyr::unnest_wider(tibble::enframe(gh_response, name = NULL), "value")
}

gh_get_html <- function(path, auth = gh::gh_token()) {
  gh_response <- gh_get(
    path,
    auth = auth,
    .accept = "application/vnd.github.v3.html"
  )
  return(gh_extract_html(gh_response))
}

gh_get_readme_html <- function(path, auth = gh::gh_token()) {
  files <- gh_get_folder(path, auth = auth)
  readme_path <- grep(
    pattern = "readme",
    files$path,
    value = TRUE,
    ignore.case = TRUE
  )
  if (length(readme_path)) {
    return(gh_get_html(readme_path, auth = auth))
  }
  cli::cli_warn(
    "No readme found in {.val {path}}.",
    class = "tt-warning-no_readme"
  )
  return(NULL)
}

# Do not hit api ----

gh_auth_check <- function(auth = gh::gh_token()) {
  if (!nchar(auth)) {
    .pkg_abort(
      c(
        x = "{.arg auth} is not a valid github token.",
        i = "See the {.vignette gh::managing-personal-access-tokens} vignette."
      ),
      "bad_gh_auth"
    )
  }
  return(auth)
}

gh_raw_to_chr <- function(encoded_raw) {
  return(rawToChar(jsonlite::base64_dec(encoded_raw)))
}

gh_extract_text <- function(gh_response) {
  if (length(gh_response) && length(gh_response$content)) {
    return(gh_raw_to_chr(gh_response$content))
  }
  .pkg_abort(
    "No content found in {.var gh_response}.",
    "bad_gh_response"
  )
}

gh_extract_html <- function(gh_response) {
  if (length(gh_response) && length(gh_response$message)) {
    return(xml2::read_html(gh_response$message))
  }
  .pkg_abort(
    "No html found in {.var gh_response}.",
    "bad_gh_response"
  )
}

gh_extract_csv <- function(gh_response, header = TRUE) {
  file_text <- gh_extract_text(gh_response)
  content <- utils::read.csv(
    text = file_text,
    header = header,
    stringsAsFactors = FALSE
  )
  attr(content, ".sha") <- gh_response$sha
  return(content)
}

gh_extract_sha_in_folder <- function(folder, file) {
  target <- purrr::keep(folder, ~ .x$name == file)
  if (length(target) && length(target[[1]]$sha)) {
    return(target[[1]]$sha)
  }
  file_names <- purrr::map_chr(folder, "name")
  .pkg_abort(
    c(
      "File {.val {file}} not found in folder.",
      i = "Found {cli::no({length(file_names)})} file{?s}: {.val {file_names}}"
    ),
    "file_not_found"
  )
}

test_that("gh_get_sha_in_folder gets sha for a file in a folder", {
  local_tt_mocked_bindings(
    gh_get = function(path, auth) {
      return(readRDS(test_path("fixtures", "static_contents.rds")))
    }
  )
  expect_equal(
    {
      gh_get_sha_in_folder("static", "tt_data_type.csv")
    },
    "5b7d51181d18d1af90caedd4e008509722612efb"
  )
  expect_equal(
    {
      gh_get_sha_in_folder("static", "tt_logo.png")
    },
    "61ab36921f82e35c621c2c4086c38b177bc89ea1"
  )
})

test_that("gh_get_csv gets and parses csv", {
  local_tt_mocked_bindings(
    gh_get = function(path, auth) {
      return(readRDS(test_path("fixtures", "tt_data_type_response.rds")))
    }
  )
  test_response <- gh_get_csv("static/tt_data_type.csv")
  expect_s3_class(test_response, "data.frame")
  expect_setequal(
    names(test_response),
    c("Week", "Date", "year", "data_files", "data_type", "delim")
  )
  expect_equal(nrow(test_response), 683)
})

test_that("gh_get_readme_html gets readme html", {
  local_tt_mocked_bindings(
    gh_get = function(path, auth, ...) {
      if (path == "data/2020") {
        return(readRDS(test_path("fixtures", "folder2020_response.rds")))
      } else if (path == "data/2020/readme.md") {
        return(readRDS(test_path("fixtures", "readme2020_response.rds")))
      }
    }
  )
  test_result <- gh_get_readme_html("data/2020")
  expected_result <- xml2::read_html(
    readRDS(test_path("fixtures", "readme2020_response.rds"))$message
  )
  expect_equal(test_result, expected_result)
})

test_that("gh_get_readme_html warns when no readme found", {
  local_tt_mocked_bindings(
    gh_get_folder = function(...) {
      list()
    }
  )
  expect_warning(
    {
      expect_null(gh_get_readme_html("data/2018/2018-04-02"))
    },
    "No readme found",
    class = "tt-warning-no_readme"
  )
})

test_that("gh_extract_text errors with empty response", {
  stbl::expect_pkg_error_snapshot(
    gh_extract_text(list()),
    "tidytuesdayR",
    "bad_gh_response"
  )
})

test_that("gh_extract_html errors with empty response", {
  stbl::expect_pkg_error_snapshot(
    gh_extract_html(list()),
    "tidytuesdayR",
    "bad_gh_response"
  )
})

test_that("gh_extract_sha_in_folder errors for missing file", {
  stbl::expect_pkg_error_snapshot(
    gh_extract_sha_in_folder(list(), "missing_file_name"),
    "tidytuesdayR",
    "file_not_found"
  )
  stbl::expect_pkg_error_snapshot(
    gh_extract_sha_in_folder(
      list(list(name = "found_file_name")),
      "missing_file_name"
    ),
    "tidytuesdayR",
    "file_not_found"
  )
})

test_that("gh_auth_check makes sure auth looks valid", {
  stbl::expect_pkg_error_snapshot(
    gh_auth_check(""),
    "tidytuesdayR",
    "bad_gh_auth"
  )
  stbl::expect_pkg_error_snapshot(
    gh_auth_check(structure("", class = "gh_pat")),
    "tidytuesdayR",
    "bad_gh_auth"
  )
  expect_no_error({
    gh_auth_check(structure("valid_token", class = "gh_pat"))
  })
})

---
title: "Curating a Dataset"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Curating a Dataset}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

We curate a new dataset (or collection of datasets) for TidyTuesday every week.
We welcome dataset submissions from the community!

There are 7 main steps to submit a dataset (plus a 0th step that you might have to do the first time you submit a dataset):

0. [Set up your GitHub account].
1. [Open these instructions as a step-by-step script].
2. [Wrangle] the data with `tt_clean()`
3. [Save & document] the data with `tt_save_dataset()`
4. [Introduce] the data with `tt_intro()`
5. [Find images] (or create them).
6. [Provide metadata] with `tt_meta()`
7. [Submit] for review with `tt_submit()`

## Set up your GitHub account

If you've never worked with GitHub, don't worry!
It's relatively easy, and we'll guide you through the setup process.
After you submit a dataset, you'll officially be an open-source contributor!

1. [Create a GitHub account](https://github.com/). It's easy and free!
2. `install.packages("usethis")` if you don't already have it.
3. Run `usethis::create_github_token()` to set up a "Personal Access Token" for GitHub. This token is how you authorize your R session to act on your behalf on GitHub. As mentioned in `usethis::create_github_token()`, you should store your token using `gitcreds::gitcreds_set()`.
3. Run `usethis::git_sitrep()`. It will print a lot of information about your git setup. You don't need to set up anything for the "GitHub project" section of that report, but check for any other issues, and resolve them as recommended by {usethis}.

## Open these instructions as a step-by-step script

These instructions are also provided as an R script.

```{r tt_curate_data}
library(tidytuesdayR)
tt_curate_data()
```

## Wrangle

We try to provide datasets as one or more tidy tibbles that can be saved as CSV files.
This ensures that the data is easy for learners to access, regardless of what tools they might use.

Fully teaching this step is beyond the scope of this vignette.
[R for Data Science](https://r4ds.hadley.nz/) is an excellent resource for help with preparation of tidy data.

However you wrangle the data, we need a single file, `cleaning.R`, to show users how you turned the raw dataset into a tidy dataset.
Use `tt_clean()` to create and open this file.

```{r tt_clean}
library(tidytuesdayR)
tt_clean()
```

By default, `cleaning.R` (and the other files we create in this process) will be created in a `tt_submission` directory in your working directory.
If you would like to create your submission in a different directory, provide a `path` argument to `tt_clean()`.

As an example of what a `cleaning.R` script should look like, imagine we want to use the built-in `state.x77` dataset from the {datasets} package.
Our `cleaning.R` file might look like this.

```{r cleaning}
library(tidyverse)
library(janitor)

states <- state.x77 |> 
  tibble::as_tibble(rownames = "state") |> 
  janitor::clean_names() |> 
  dplyr::mutate(
    dplyr::across(
      c("population", "income", "frost", "area"),
      as.integer
    )
  )
```

Please `library()` any packages used in your script at the top of the script to make it easier for learners to see what they might need to install.
In addition to the {tidyverse}, we often use {janitor} to clean up column names.

While not required, we usually try to cast numeric variables to their "actual" class (integer, logical, date, etc, not necessarily leaving them as the "double" class that many datasets use by default).
That way users can see a little more information about what they should expect in that column.

Be sure to execute your code (ideally in a fresh R session to ensure that it works as expected) before moving on to the [Save & document] step.

## Save & document

Use `tt_save_dataset()` to save each wrangled dataset as a csv file, and to create a formatted dictionary markdown file for each dataset.

```{r tt_save_dataset}
tt_save_dataset(states)
```

Running that code creates two files in your `tt_submission` directory: `states.csv` and `states.md`.
The `.md` file contains a formatted data dictionary table for the dataset, and is opened so you can provide information about each column.

```states.md
|variable   |class     |description                           |
|:----------|:---------|:-------------------------------------|
|state      |character |Describe this field in sentence case. |
|population |integer   |Describe this field in sentence case. |
|income     |integer   |Describe this field in sentence case. |
|illiteracy |double    |Describe this field in sentence case. |
|life_exp   |double    |Describe this field in sentence case. |
|murder     |double    |Describe this field in sentence case. |
|hs_grad    |double    |Describe this field in sentence case. |
|frost      |integer   |Describe this field in sentence case. |
|area       |integer   |Describe this field in sentence case. |
```

Provide information so users can understand what each column represents.

```states.md
|variable   |class     |description                           |
|:----------|:---------|:-------------------------------------|
|state      |character |Name of the state. |
|population |character |Population estimate as of July 1, 1975. |
|income     |integer   |Per capita income in 1974. |
|illiteracy |integer   |Illiteracy in 1970, as percent of population. |
|life_exp   |double    |Life expectancy in years (1969–71). |
|murder     |double    |Murder and non-negligent manslaughter rate per 100,000 population in 1976. |
|hs_grad    |double    |Percent high-school graduates in 1970. |
|frost      |integer   |Mean number of days with minimum temperature below freezing (1931–1960) in capital or large city. |
|area       |integer   |Land area in square miles. |
```

Notice that we don't have to perfectly align the `|` characters in the table.

Repeat this process for each dataset that you wrangled.

## Introduce

Next we need to introduce the dataset.
Create an `intro.md` file in your `tt_submission` directory with `tt_intro()`.

```{r tt_intro}
tt_intro()
```

The file looks like this:

```
<!-- 
Describe the dataset. See previous weeks for the general format of the
description. The description is the part of the readme.md file above "The Data";
everything else will be filled in from the other md files in this directory +
automatic scripts. We usually include brief introduction along the lines of
"This week we're exploring DATASET" or "The dataset this week comes from 
SOURCE".
-->

<!-- Add a quote from the source, starting lines with a ">" character, like 
this:
> Plant traits are critical to plant form and function — including growth, 
> survival and reproduction — and therefore shape fundamental aspects of
> population and ecosystem dynamics as well as ecosystem services. Here, we 
> present a global species-level compilation of key functional traits for palms 
> (Arecaceae), a plant family with keystone importance in tropical and 
> subtropical ecosystems.
-->

> PasteQuoteHere

<!--
Optional: Add questions that users should try to answer. For example:
- How does the sizes of the different species of palms vary across sub families?
- Which fruit colors occur most often?
-->
```

Fill in a description with information about the dataset, as described in the comments (text between `<!--` and `-->`). 
Find a quotation from the source or the article that describes the dataset or a use case for it, and paste it over `PasteQuoteHere`. 
Optionally, add one or more questions that participants might answer using the data.
For our `states` dataset, we might end up with something like this:
```
This week we're exploring data about the 50 states of the United States of America!
The data this week comes from the U.S. Department of Commerce, Bureau of the Census (1977) *Statistical Abstract of the United States* and *County and City Data Book* via the core R {datasets} package.

> Data sets related to the 50 states of the United States of America. Note that all data are arranged according to alphabetical order of the state names.

- Is there a relationship between income and illiteracy?
- Which states had the highest population density in 1975? Which states had the lowest?
```

## Find images

We need at least one image to include in our social media posts about the dataset.
These images might come from the source of the dataset, or you might create them yourself.
For our `states` data, we'll create a basic map of the united states with the states colored by population.

```{r states-population}
library(tidyverse)
state_outlines <- ggplot2::map_data("state")
states |> 
  dplyr::select(state, population) |>
  dplyr::mutate(region = tolower(state)) |> 
  dplyr::left_join(state_outlines, by = "region") |> 
  ggplot(aes(long, lat, group = state, fill = population)) +
  geom_polygon() +
  coord_map() +
  theme_void()
ggsave(
  usethis::proj_path("tt_submission", "states_population.png"),
  width = 5, height = 3, units = "in",
  bg = "white"
)
```

## Provide metadata

The final preparation step is to provide metadata about the dataset.
Create a `meta.yaml` file in your `tt_submission` directory with `tt_meta()`.

To avoid entering your attribution information every time you submit a dataset, we recommend adding your details to your `.Rprofile`.
Open your `.Rprofile` with `usethis::edit_r_profile()`, then add an options block like this:

```r
# tidytuesdayR attribution
options(
  tidytuesdayR.attribution = "Jon Harmon, Data Science Learning Community",
  tidytuesdayR.bluesky = "jonthegeek.com",
  tidytuesdayR.linkedin = "jonthegeek",
  tidytuesdayR.mastodon = "fosstodon.org/@jonthegeek"
)
```

After saving and restarting R, `tt_meta()` will use these values as defaults.

```{r tt-meta}
tt_meta(
  title = "The 50 US States",
  article_title = "U.S. Department of Commerce, Bureau of the Census",
  article_url = "https://www.census.gov/",
  source_title = "The R datasets package",
  source_url = "https://www.r-project.org/",
  image_filename = "states_population.png",
  image_alt = "A map of the continental United States, with each state colored in shades of blue by population as of 1975. California and New York are the lightest, indicating the highest population. Maine, New Hampshire, Vermont, and the Plains States are all quite dark, indicating low population.",
  attribution = "Jon Harmon, Data Science Learning Community",
  github = "jonthegeek",
  bluesky = "jonthegeek.com",
  linkedin = "jonthegeek",
  mastodon = "fosstodon.org/@jonthegeek"
)
```

In an interactive session, the function will ask you questions to pre-fill the metadata file, and then open it so you can confirm that everything looks right.
You can also provide the data directly to the function (as I did here), which will skip the Q&A. Notice that the social media links are formatted as `@username` in all cases except Mastodon, which requires `@username@server`.

The completed `meta.yaml` file looks like this (I edited the image alt text so I could read it without scrolling, but otherwise this is what `tt_meta()` produced):
```yaml
title: "The 50 US States"
article:
  title: "U.S. Department of Commerce, Bureau of the Census"
  url: "https://www.census.gov/"
data_source:
  title: "The R datasets package"
  url: "https://www.r-project.org/"
images:
# Please include at least one image, and up to three images
- file: "states_population.png"
  alt: >
    A map of the continental United States, with each state colored in shades of
    blue by population as of 1975. California and New York are the lightest,
    indicating the highest population. Maine, New Hampshire, Vermont, and the
    Plains States are all quite dark, indicating low population.
credit:
  post: "Jon Harmon, Data Science Learning Community"
  github: "@jonthegeek"
  bluesky: "@jonthegeek.com"
  linkedin: "@jonthegeek"
  mastodon: "@jonthegeek@fosstodon.org"
```

## Submit

Once your submission is prepared, please submit it for review!
If you haven't already done so, be sure to [Set up your GitHub account].
Once that's ready, use `tt_submit()` to submit your dataset as a pull request (PR) on the TidyTuesday GitHub repo.
A PR is a way to suggest changes to a repository (a collection of files on GitHub), and it's how we manage submissions to TidyTuesday.
You're requesting that we "pull" changes from your copy on GitHub to our source repository.

**Note:** To avoid confusion, we recommend that you [login to GitHub.com](https://github.com/login) before you run this command.

```{r tt_submit}
tt_submit()
```

If we request that you make changes, you can use `tt_submit()` again to upload any files you change.

After confirming that everything looks correct, we'll assign your submission to a date.
Congratulations! You are now an open-source contributor!

---
title: "TidyTemplate"
date: {{{call_date}}}
output: html_document
---

# TidyTuesday

Join the Data Science Learning Community in the weekly #TidyTuesday event!
Every week we post a raw dataset, a chart or article related to that dataset, and ask you to explore the data.
While the dataset will be “tamed”, it will not always be tidy! As such you might need to apply various R for Data Science techniques to wrangle the data into a true tidy format.
The goal of TidyTuesday is to apply your R skills, get feedback, explore other’s work, and connect with the greater #RStats community!
As such we encourage everyone of all skills to participate!

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

```{r libraries, message=FALSE, warning=FALSE}
library(tidyverse)
library(tidytuesdayR)
```

# Load the weekly Data

Download the weekly data and make available in the `tt` object.

```{r Load}
tt <- tt_load("{{{call_tuesday}}}")
```


# Readme

Take a look at the readme for the weekly data to get insight on the dataset.
This includes a data dictionary, source, and a link to an article on the data.

```{r Readme, eval = interactive()}
tt
```


# Glimpse Data

Take an initial look at the format of the data available.

```{r Glimpse}
map(tt, glimpse)
```

# Wrangle

Explore the data and process it into a nice format for plotting! Access each dataset by name by using a dollarsign after the `tt` object and then the name of the data set.

```{r Wrangle}



```


# Visualize

Using your processed dataset, create your unique visualization.

```{r Visualize}


  
```

# Save Image

Save your image for sharing. Be sure to use the `#TidyTuesday` hashtag in your post on twitter! 

```{r}
# This will save your most recent plot
ggsave(
  filename = "My TidyTuesday Plot.png",
  device = "png"
)
```

<!-- 
Describe the dataset. See previous weeks for the general format of the
description. The description is the part of the readme.md file above "The Data";
everything else will be filled in from the other md files in this directory +
automatic scripts. We usually include brief introduction along the lines of
"This week we're exploring DATASET" or "The dataset this week comes from 
SOURCE".
-->

<!-- Add a quote from the source, starting lines with a ">" character, like 
this:
> Plant traits are critical to plant form and function — including growth, 
> survival and reproduction — and therefore shape fundamental aspects of
> population and ecosystem dynamics as well as ecosystem services. Here, we 
> present a global species-level compilation of key functional traits for palms 
> (Arecaceae), a plant family with keystone importance in tropical and 
> subtropical ecosystems.
-->

> PasteQuoteHere

<!--
Optional: Add questions that users should try to answer. For example:
- How does the sizes of the different species of palms vary across sub families?
- Which fruit colors occur most often?
-->

This week we're exploring data about the 50 states of the United States of America!
The data this week comes from the U.S. Department of Commerce, Bureau of the Census (1977) *Statistical Abstract of the United States* and *County and City Data Book* via the core R {datasets} package.

> Data sets related to the 50 states of the United States of America. Note that all data are arranged according to alphabetical order of the state names.

- Is there a relationship between income and illiteracy?
- Which states had the highest population density in 1975? Which states had the lowest?

|variable   |class     |description                           |
|:----------|:---------|:-------------------------------------|
|state      |character |Name of the state. |
|population |character |Population estimate as of July 1, 1975. |
|income     |integer   |Per capita income in 1974. |
|illiteracy |integer   |Illiteracy in 1970, as percent of population. |
|life_exp   |double    |Life expectancy in years (1969–71). |
|murder     |double    |Murder and non-negligent manslaughter rate per 100,000 population in 1976. |
|hs_grad    |double    |Percent high-school graduates in 1970. |
|frost      |integer   |Mean number of days with minimum temperature below freezing (1931–1960) in capital or large city. |
|area       |integer   |Land area in square miles. |

# ensure_arg_filled errors informatively

    Code
      (expect_pkg_error_classes(ensure_arg_filled("", question = "", arg_name = "myArg"),
      "tidytuesdayR", "required_arg"))
    Output
      <error/tidytuesdayR-error-required_arg>
      Error:
      ! `myArg` is required.

# format_image_data errors informatively

    Code
      (expect_pkg_error_classes(format_image_data(c("a", "b"), "c"), "tidytuesdayR",
      "image_data"))
    Output
      <error/tidytuesdayR-error-image_data>
      Error in `format_image_data()`:
      ! Please provide at least one image filename and corresponding alt text.

# ensure_arg_filled errors informatively for NULL (#142)

    Code
      (expect_pkg_error_classes(ensure_arg_filled(NULL, question = "", arg_name = "myArg"),
      "tidytuesdayR", "required_arg"))
    Output
      <error/tidytuesdayR-error-required_arg>
      Error:
      ! `myArg` is required.

# tt_find_dataset_files errors informatively for extra files

    Code
      (expect_pkg_error_classes(tt_find_dataset_files(test_path("fixtures",
        "tt_submission_extra")), "tidytuesdayR", "extra_files"))
    Output
      <error/tidytuesdayR-error-extra_files>
      Error in `tt_find_dataset_files()`:
      ! `path` should only contain submission files.
      x Extra files: fixtures/tt_submission_extra/extra.R

# tt_find_dataset_files errors informatively for missing files

    Code
      (expect_pkg_error_classes(tt_find_dataset_files(test_path("fixtures",
        "tt_submission_missing")), "tidytuesdayR", "missing_expected"))
    Output
      <error/tidytuesdayR-error-missing_expected>
      Error in `tt_find_expected_files()`:
      ! All expected files must exist in `path`.
      x Missing files: 'fixtures/tt_submission_missing/cleaning.R'

# tt_find_dataset_files errors informatively for missing images

    Code
      (expect_pkg_error_classes(tt_find_dataset_files(test_path("fixtures",
        "tt_submission_missing_image1")), "tidytuesdayR", "missing_images"))
    Output
      <error/tidytuesdayR-error-missing_images>
      Error in `tt_find_images()`:
      ! All images in meta.yaml must exist in `path`.
      x Missing images: 'fixtures/tt_submission_missing_image1/states_population.png'

---

    Code
      (expect_pkg_error_classes(tt_find_dataset_files(test_path("fixtures",
        "tt_submission_missing_image2")), "tidytuesdayR", "no_images"))
    Output
      <error/tidytuesdayR-error-no_images>
      Error in `tt_find_images()`:
      ! No images found in meta.yaml

# tt_find_dataset_files errors informatively for missing dictionary

    Code
      (expect_pkg_error_classes(tt_find_dataset_files(test_path("fixtures",
        "tt_submission_missing_md")), "tidytuesdayR", "missing_dictionaries"))
    Output
      <error/tidytuesdayR-error-missing_dictionaries>
      Error in `tt_find_dictionaries()`:
      ! All datasets must have an associated md file in `path`.
      x Missing dictionaries: 'fixtures/tt_submission_missing_md/states.md'

# tt_find_csv_files errors when CSV exceeds 25MB

    Code
      (expect_pkg_error_classes(tt_find_csv_files(test_path("fixtures",
        "tt_submission")), "tidytuesdayR", "csv_size"))
    Output
      <error/tidytuesdayR-error-csv_size>
      Error in `tt_validate_csv_sizes()`:
      ! CSV files must be <= 25MB to upload to GitHub.
      x Files too large:
      * fixtures/tt_submission/states.csv (30M)

# tt_find_csv_files handles multiple large CSVs

    Code
      (expect_pkg_error_classes(tt_find_csv_files(test_path("fixtures",
        "tt_submission")), "tidytuesdayR", "csv_size"))
    Output
      <error/tidytuesdayR-error-csv_size>
      Error in `tt_validate_csv_sizes()`:
      ! CSV files must be <= 25MB to upload to GitHub.
      x Files too large:
      * fixtures/tt_submission/file1.csv (31457280)
      * fixtures/tt_submission/file2.csv (41943040)

|variable |class  |description                           |
|:--------|:------|:-------------------------------------|
|mpg      |double |Describe this field in sentence case. |
|cyl      |double |Describe this field in sentence case. |
|disp     |double |Describe this field in sentence case. |
|hp       |double |Describe this field in sentence case. |
|drat     |double |Describe this field in sentence case. |
|wt       |double |Describe this field in sentence case. |
|qsec     |double |Describe this field in sentence case. |
|vs       |double |Describe this field in sentence case. |
|am       |double |Describe this field in sentence case. |
|gear     |double |Describe this field in sentence case. |
|carb     |double |Describe this field in sentence case. |

# last_tuesday errors with bad dates

    Code
      (expect_pkg_error_classes(last_tuesday("blue"), "tidytuesdayR", "invalid_date"))
    Output
      <error/tidytuesdayR-error-invalid_date>
      Error in `last_tuesday()`:
      ! "blue" cannot be coerced to a <Date>.

---

    Code
      (expect_pkg_error_classes(last_tuesday(1), "tidytuesdayR", "invalid_date"))
    Output
      <error/tidytuesdayR-error-invalid_date>
      Error in `last_tuesday()`:
      ! 1 cannot be coerced to a <Date>.

# tt_download errors for bad file

    Code
      (expect_pkg_error_classes(tt_download(tt, "bad_filename"), "tidytuesdayR",
      "bad_file"))
    Output
      <error/tidytuesdayR-error-bad_file>
      Error in `tt_download()`:
      ! `files` must be one or more of "agencies.csv" or "launches.csv", or "All".

# .pkg_abort() throws a classed tidytuesdayR error (#149)

    Code
      (expect_pkg_error_classes(.pkg_abort("This is a test error.", "test_subclass"),
      "tidytuesdayR", "test_subclass"))
    Output
      <error/tidytuesdayR-error-test_subclass>
      Error:
      ! This is a test error.

# Close dates are suggested if provided date is incorrect

    Code
      (expect_pkg_error_classes(tt_check_date("2019-04-04"), "tidytuesdayR",
      "invalid_date"))
    Output
      <error/tidytuesdayR-error-invalid_date>
      Error in `tt_check_date.date()`:
      ! 2019-04-04 does not have TidyTuesday data.
      i Did you mean 2019-04-02?

# Invalid weeks throw errors

    Code
      (expect_pkg_error_classes(tt_check_date(2018, 20), "tidytuesdayR",
      "invalid_date"))
    Output
      <error/tidytuesdayR-error-invalid_date>
      Error in `tt_check_date.year()`:
      ! Week 20 does not have TidyTuesday data in 2018.
      i Please choose a valid week from 1-19, 21-38

---

    Code
      (expect_pkg_error_classes(tt_check_date(2018, 0), "tidytuesdayR",
      "invalid_date"))
    Output
      <error/tidytuesdayR-error-invalid_date>
      Error in `tt_check_date.year()`:
      ! `week` must be a valid positive integer value.

---

    Code
      (expect_pkg_error_classes(tt_check_date(2018, 7), "tidytuesdayR",
      "invalid_date"))
    Output
      <error/tidytuesdayR-error-invalid_date>
      Error in `tt_check_date.year()`:
      ! The dataset for 2018 week 7 is dirty and cannot be automatically loaded.

---

    Code
      (expect_pkg_error_classes(tt_check_date(2018, 8), "tidytuesdayR",
      "invalid_date"))
    Output
      <error/tidytuesdayR-error-invalid_date>
      Error in `tt_check_date.year()`:
      ! The dataset for 2018 week 8 is dirty and cannot be automatically loaded.

---

    Code
      (expect_pkg_error_classes(tt_check_date(2020, 1), "tidytuesdayR",
      "invalid_date"))
    Output
      <error/tidytuesdayR-error-invalid_date>
      Error in `tt_check_date.year()`:
      ! Week 1 of 2020 does not have data available for download.

# invalid entries are flagged

    Code
      (expect_pkg_error_classes(tt_check_date("xyz"), "tidytuesdayR", "invalid_date"))
    Output
      <error/tidytuesdayR-error-invalid_date>
      Error in `tt_check_date()`:
      ! Entries must render to a valid date or year

# tt_check_year checks years

    Code
      (expect_pkg_error_classes(tt_check_year(2015), "tidytuesdayR", "invalid_year"))
    Output
      <error/tidytuesdayR-error-invalid_year>
      Error in `tt_check_year()`:
      ! TidyTuesday did not exist in 2015 (or 2015 is in the future).
      i Available years: 2024, 2023, 2022, 2021, 2020, 2019, and 2018

# tt_check_date errors informatively with no args

    Code
      (expect_pkg_error_classes(tt_check_date(), "tidytuesdayR", "invalid_date"))
    Output
      <error/tidytuesdayR-error-invalid_date>
      Error in `tt_check_date()`:
      ! Provide either the year & week or the date of the TidyTuesday dataset.

# tt_check_date errors informatively for the dirtiest dataset

    Code
      (expect_pkg_error_classes(tt_check_date("2018-05-15"), "tidytuesdayR",
      "invalid_date"))
    Output
      <error/tidytuesdayR-error-invalid_date>
      Error in `tt_check_date.date()`:
      ! The dataset for 2018-05-15 is dirty and cannot be automatically loaded.

# gh_extract_text errors with empty response

    Code
      (expect_pkg_error_classes(gh_extract_text(list()), "tidytuesdayR",
      "bad_gh_response"))
    Output
      <error/tidytuesdayR-error-bad_gh_response>
      Error in `gh_extract_text()`:
      ! No content found in `gh_response`.

# gh_extract_html errors with empty response

    Code
      (expect_pkg_error_classes(gh_extract_html(list()), "tidytuesdayR",
      "bad_gh_response"))
    Output
      <error/tidytuesdayR-error-bad_gh_response>
      Error in `gh_extract_html()`:
      ! No html found in `gh_response`.

# gh_extract_sha_in_folder errors for missing file

    Code
      (expect_pkg_error_classes(gh_extract_sha_in_folder(list(), "missing_file_name"),
      "tidytuesdayR", "file_not_found"))
    Output
      <error/tidytuesdayR-error-file_not_found>
      Error in `gh_extract_sha_in_folder()`:
      ! File "missing_file_name" not found in folder.
      i Found no files:

---

    Code
      (expect_pkg_error_classes(gh_extract_sha_in_folder(list(list(name = "found_file_name")),
      "missing_file_name"), "tidytuesdayR", "file_not_found"))
    Output
      <error/tidytuesdayR-error-file_not_found>
      Error in `gh_extract_sha_in_folder()`:
      ! File "missing_file_name" not found in folder.
      i Found 1 file: "found_file_name"

# gh_auth_check makes sure auth looks valid

    Code
      (expect_pkg_error_classes(gh_auth_check(""), "tidytuesdayR", "bad_gh_auth"))
    Output
      <error/tidytuesdayR-error-bad_gh_auth>
      Error in `gh_auth_check()`:
      x `auth` is not a valid github token.
      i See the `vignette(gh::managing-personal-access-tokens)` vignette.

---

    Code
      (expect_pkg_error_classes(gh_auth_check(structure("", class = "gh_pat")),
      "tidytuesdayR", "bad_gh_auth"))
    Output
      <error/tidytuesdayR-error-bad_gh_auth>
      Error in `gh_auth_check()`:
      x `auth` is not a valid github token.
      i See the `vignette(gh::managing-personal-access-tokens)` vignette.

# tt_datasets throws errors when asking for invalid years

    Code
      (expect_pkg_error_classes(tt_datasets(2017), "tidytuesdayR", "invalid_year"))
    Output
      <error/tidytuesdayR-error-invalid_year>
      Error in `tt_check_year()`:
      ! TidyTuesday did not exist in 2017 (or 2017 is in the future).
      i Available years: 2018, 2019, 2020, 2021, 2022, 2023, and 2024

# printing tt_datasets returns all the values as a printed data.frame if not interactive

    Code
      print(ds, is_interactive = FALSE)
    Output
        a b
      1 1 3
      2 2 4

# printing tt_available returns all the values as a printed data.frame if not interactive

    Code
      print(ds, is_interactive = FALSE)
    Output
      Year: test_year
      
        a b
      1 1 3
      2 2 4
      
      

# tt_load_gh errors when incorrect date

    Code
      (expect_pkg_error_classes(tt_load_gh("2019-01-16"), "tidytuesdayR",
      "invalid_date"))
    Output
      <error/tidytuesdayR-error-invalid_date>
      Error in `tt_check_date.date()`:
      ! 2019-01-16 does not have TidyTuesday data.
      i Did you mean 2019-01-15?

# print.tt lists all the available files for the weeks tt

    Code
      test_result <- print(tt)
    Message
      Available datasets in this TidyTuesday:
      	agencies.csv 
      	launches.csv 
      	

# tt_download_file errors for bad index

    Code
      (expect_pkg_error_classes(tt_download_file(tt, 3), "tidytuesdayR", "bad_index"))
    Output
      <error/tidytuesdayR-error-bad_index>
      Error:
      ! File 3 not found in the available files for 2019-01-15.

---

    Code
      (expect_pkg_error_classes(tt_download_file(tt, "bad_filename"), "tidytuesdayR",
      "bad_index"))
    Output
      <error/tidytuesdayR-error-bad_index>
      Error:
      ! File bad_filename not found in the available files for 2019-01-15.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{readme}
\alias{readme}
\title{Readme HTML maker and Viewer}
\usage{
readme(tt)
}
\arguments{
\item{tt}{tt_data object for printing}
}
\value{
Null, invisibly. Used to show readme of the downloaded TidyTuesday
dataset in the Viewer.
}
\description{
Readme HTML maker and Viewer
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
if (rate_limit_check(quiet = TRUE) > 30) {
  tt_output <- tt_load_gh("2019-01-15")
  readme(tt_output)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_download_file.R
\name{tt_download_file}
\alias{tt_download_file}
\title{Download a TidyTuesday dataset file}
\usage{
tt_download_file(tt, x, ..., auth = gh::gh_token())
}
\arguments{
\item{tt}{A \code{tt} object, output from \code{\link[=tt_load_gh]{tt_load_gh()}}.}

\item{x}{Index or name of file to download.}

\item{...}{Additional parameters to pass to the parsing functions. Note:
These arguments will be passed for all filetypes.}

\item{auth}{A GitHub token. See \code{\link[gh:gh_token]{gh::gh_token()}} for more details.}
}
\value{
tibble containing the contents of the file downloaded from git
}
\description{
Download an actual data file from the TidyTuesday github repository.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
tt_gh <- tt_load_gh("2019-01-15")

agencies <- tt_download_file(tt_gh, 1)
launches <- tt_download_file(tt_gh, "launches.csv")
\dontshow{\}) # examplesIf}
}
\concept{tt_download_file}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tidytuesdayR-package.R
\docType{package}
\name{tidytuesdayR-package}
\alias{tidytuesdayR}
\alias{tidytuesdayR-package}
\title{tidytuesdayR: Access the Weekly 'TidyTuesday' Project Dataset}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

'TidyTuesday' is a project by the 'Data Science Learning Community' in which they post a weekly dataset in a public data repository (\url{https://github.com/rfordatascience/tidytuesday}) for people to analyze and visualize. This package provides the tools to easily download this data and the description of the source.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://dslc-io.github.io/tidytuesdayR/}
  \item \url{https://github.com/dslc-io/tidytuesdayR}
  \item Report bugs at \url{https://github.com/dslc-io/tidytuesdayR/issues}
}

}
\author{
\strong{Maintainer}: Jon Harmon \email{jonthegeek@gmail.com} (\href{https://orcid.org/0000-0003-4781-4346}{ORCID})

Authors:
\itemize{
  \item Ellis Hughes \email{ellishughes@live.com}
}

Other contributors:
\itemize{
  \item Thomas Mock \email{j.thomasmock@gmail.com} [contributor]
  \item Data Science Learning Community \email{tidytuesday@dslc.io} [data contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_load.R, R/tt_load_gh.R, R/utils.R
\name{print.tt_data}
\alias{print.tt_data}
\alias{print.tt}
\alias{printing}
\title{print methods of the tt objects}
\usage{
\method{print}{tt_data}(x, ...)

\method{print}{tt}(x, ...)
}
\arguments{
\item{x}{a tt_data or tt object}

\item{...}{further arguments passed to or from other methods.}
}
\value{
\code{x}, invisibly.
}
\description{
In tidytuesdayR there are nice print methods for the objects that were used
to download and store the data from the TidyTuesday repo. They will always
print the available datasets/files. If there is a readme available,
it will try to display the TidyTuesday readme.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
tt <- tt_load_gh("2019-01-15")
print(tt)

tt_data <- tt_download(tt, files = "All")
print(tt_data)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa-conditions.R
\name{.pkg_abort}
\alias{.pkg_abort}
\title{Raise a package-scoped error}
\usage{
.pkg_abort(
  message,
  subclass,
  call = rlang::caller_env(),
  message_env = rlang::caller_env(),
  ...
)
}
\arguments{
\item{message}{(\code{character}) The message for the new error. Messages will be
formatted with \code{\link[cli:cli_bullets]{cli::cli_bullets()}}.}

\item{subclass}{(\code{character}) Class(es) to assign to the error. Will be
prefixed by "\{package\}-error-".}

\item{call}{(\code{environment}) The caller environment for error messages.}

\item{message_env}{(\code{environment}) The execution environment to use to
evaluate variables in error messages.}

\item{...}{Additional parameters passed to \code{\link[cli:cli_abort]{cli::cli_abort()}} and on to
\code{\link[rlang:abort]{rlang::abort()}}.}
}
\value{
Does not return.
}
\description{
Raise a package-scoped error
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_curate_data.R
\name{tt_curate_data}
\alias{tt_curate_data}
\title{Guidance for TidyTuesday dataset curation}
\usage{
tt_curate_data(open = rlang::is_interactive())
}
\arguments{
\item{open}{Whether to open the file for interactive editing.}
}
\value{
The path to the \code{tt_curation.R} script, invisibly.
}
\description{
Open an R script to guide you through the process of curating and submitting
a TidyTuesday dataset. See \verb{vignette("curating", package = "tidytuesdayR)}
for more information.
}
\examples{
tt_curate_data()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_download.R
\name{tt_download}
\alias{tt_download}
\title{Download TidyTuesday data}
\usage{
tt_download(tt, files = "All", ..., auth = gh::gh_token())
}
\arguments{
\item{tt}{A \code{tt} object, output from \code{\link[=tt_load_gh]{tt_load_gh()}}.}

\item{files}{Which file names to download. Default "All" downloads all files
for the specified week.}

\item{...}{Additional parameters to pass to the parsing functions. Note:
These arguments will be passed for all filetypes.}

\item{auth}{A GitHub token. See \code{\link[gh:gh_token]{gh::gh_token()}} for more details.}
}
\value{
A list of tibbles from the downloaded files.
}
\description{
Download all or specific files identified in a TidyTuesday dataset.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
# Get the list of files for a week.
tt_output <- tt_load_gh("2019-01-15")

# Download a specific file.
agencies <- tt_download(tt_output, files = "agencies.csv")
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_load_gh.R
\name{tt_compile}
\alias{tt_compile}
\title{Get TidyTuesday readme and list of files and HTML based on the date}
\usage{
tt_compile(date, auth = gh::gh_token())
}
\arguments{
\item{date}{date of TidyTuesday of interest}

\item{auth}{A GitHub token. See \code{\link[gh:gh_token]{gh::gh_token()}} for more details.}
}
\description{
Get TidyTuesday readme and list of files and HTML based on the date
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_curate_utils.R
\name{prep_tt_curate}
\alias{prep_tt_curate}
\title{Set up a directory for dataset curation}
\usage{
prep_tt_curate(
  path = "tt_submission",
  ignore = FALSE,
  env = rlang::caller_env()
)
}
\arguments{
\item{path}{The relative path to the directory to hold your submission files
(\code{tt_submission} by default). If this directory does not exist, it will be
created.}

\item{ignore}{Should the newly created file be added to \code{.Rbuildignore}?}
}
\value{
The resolved path (invisibly).
}
\description{
Set up a directory for dataset curation
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/github_api.R
\name{gh_get}
\alias{gh_get}
\title{Get data from the tt github repo.}
\usage{
gh_get(path, auth = gh::gh_token(), ...)
}
\arguments{
\item{path}{Path within the \code{rfordatascience/tidytuesday} repo.}

\item{auth}{A GitHub token. See \code{\link[gh:gh_token]{gh::gh_token()}} for more details.}

\item{...}{Additional parameters passed to \code{\link[gh:gh]{gh::gh()}}.}
}
\value{
The GitHub response as parsed by \code{\link[gh:gh]{gh::gh()}}.
}
\description{
Get data from the tt github repo.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_submit.R
\name{tt_submit}
\alias{tt_submit}
\title{Submit a TidyTuesday dataset}
\usage{
tt_submit(
  path = "tt_submission",
  auth = gh::gh_token(),
  open = rlang::is_interactive()
)
}
\arguments{
\item{path}{The relative path to the directory to hold your submission files
(\code{tt_submission} by default). If this directory does not exist, it will be
created.}

\item{auth}{A GitHub token. See \code{\link[gh:gh_token]{gh::gh_token()}} for more details.}

\item{open}{Whether to open the pull request in a browser. Defaults to \code{TRUE}
in an interactive session.}
}
\value{
The URL of the pull request, invisibly.
}
\description{
Submit a curated dataset for review by uploading it to GitHub and creating a
pull request. The dataset should be prepared using \code{\link[=tt_clean]{tt_clean()}},
\code{\link[=tt_save_dataset]{tt_save_dataset()}}, \code{\link[=tt_intro]{tt_intro()}}, and \code{\link[=tt_meta]{tt_meta()}}. You can also use this
function to submit changes to your local copies of the files.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
# First set up a dataset in the "tt_submission" folder.
tt_submit()
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/last_tuesday.R
\name{last_tuesday}
\alias{last_tuesday}
\title{Find the most recent tuesday}
\usage{
last_tuesday(date = today(tzone = "America/New_York"))
}
\arguments{
\item{date}{A date as a date object or character string in \code{YYYY-MM-DD}
format. Defaults to today's date.}
}
\value{
The TidyTuesday date in the same week as the specified date, using
Monday as the start of the week.
}
\description{
Identify the most recent 'TidyTuesday' date relative to a specified date.
}
\examples{
last_tuesday() # get last Tuesday relative to today's date
last_tuesday("2020-01-01") # get last Tuesday relative to a specified date

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_save_dataset.R
\name{tt_save_dataset}
\alias{tt_save_dataset}
\title{Save datasets for submission}
\usage{
tt_save_dataset(
  dataset,
  path = "tt_submission",
  dataset_name = rlang::caller_arg(dataset),
  open = rlang::is_interactive(),
  ignore = FALSE
)
}
\arguments{
\item{dataset}{The clean dataset to save. The dataset must be a data.frame.}

\item{path}{The relative path to the directory to hold your submission files
(\code{tt_submission} by default). If this directory does not exist, it will be
created.}

\item{dataset_name}{The name to save the dataset as. By default, the name of
the dataset variable is used.}

\item{open}{Open the newly created file for editing? Happens in RStudio, if
applicable, or via \code{\link[utils:file.edit]{utils::file.edit()}} otherwise.}

\item{ignore}{Should the newly created file be added to \code{.Rbuildignore}?}
}
\value{
A logical vector indicating whether the file was created or
modified, invisibly.
}
\description{
Datasets for TidyTuesday submissions should be saved in a specific format,
with an accompanying data dictionary \code{dataset_name.md} file. This function
saves the dataset as a CSV file in your submission directory (creating the
submission directory if it does not already exist), and creates a data
dictionary file for you to fill out. If you're in an interactive session, the
dictionary file is opened for editing.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}

  tt_save_dataset(mtcars)
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_meta.R
\name{tt_meta}
\alias{tt_meta}
\title{Create and open meta.yaml}
\usage{
tt_meta(
  path = "tt_submission",
  title,
  article_title,
  article_url,
  source_title,
  source_url,
  image_filename,
  image_alt,
  attribution = getOption("tidytuesdayR.attribution"),
  github = gh::gh_whoami()$login,
  bluesky = getOption("tidytuesdayR.bluesky"),
  linkedin = getOption("tidytuesdayR.linkedin"),
  mastodon = getOption("tidytuesdayR.mastodon"),
  open = rlang::is_interactive(),
  ignore = FALSE
)
}
\arguments{
\item{path}{The relative path to the directory to hold your submission files
(\code{tt_submission} by default). If this directory does not exist, it will be
created.}

\item{title}{A short title for your submission. It should fit into the
sentence "This week we're exploring \code{title}!" For example, for "This week
we're exploring The 50 US States!", the \code{title} would be \code{"The 50 US States"}.}

\item{article_title}{The title of an article or other website that has
something to do with the data. This should usually be an article that uses
or describes the dataset, but any related website is acceptable.}

\item{article_url}{The URL of the article whose title is \code{article_title}.}

\item{source_title}{The title of the source of the dataset. This is usually a
website, but might be an R package or a journal article, for example.}

\item{source_url}{A URL associated with the source. Ideally this should be a
URL where users can download the data, but, if that isn't possible, provide
a URL that is somehow related to the source of the data.}

\item{image_filename}{A character vector with at least one file name for an
image to accompany the post. This might be a plot of the data, or some othe
image somehow connected to the data.}

\item{image_alt}{Text that can take the place of the image for a visually
impaired user or anybody else who cannot see the image. Don't just say "A
plot of the data", but rather describe what information you can glean from
the plot, such as "A map of the continental United States, with each state
colored in shades of blue by population as of 1975. California and New York
are the lightest, indicating the highest population. Maine, New Hampshire,
Vermont, and the Plains States are all quite dark, indicating low
population."}

\item{attribution}{Your name as you would like it to appear when we credit
you in the post for this dataset. You can include a title and/or
affiliation if you like, such as "Jon Harmon, Executive Director, Data
Science Learning Community". Defaults to
\code{getOption("tidytuesdayR.attribution")}. We recommend setting this option
in your \code{.Rprofile} with \code{usethis::edit_r_profile()} so you don't have to
provide it every time you submit a dataset.}

\item{github}{Your GitHub username, or a link to your profile on GitHub.}

\item{bluesky}{Your Bluesky username, or a link to your profile on Bluesky.
Leave as \code{NULL} if you do not wish to be credited on Bluesky. Defaults to
\code{getOption("tidytuesdayR.bluesky")}.}

\item{linkedin}{Your LinkedIn username, or a link to your profile on LinkedIn.
Leave as \code{NULL} if you do not wish to be credited on LinkedIn. Defaults to
\code{getOption("tidytuesdayR.linkedin")}.}

\item{mastodon}{Your mastodon server and username, or a link to your profile
on a mastodon server. Leave as \code{NULL} if you do not wish to be credited on
Mastodon. Defaults to \code{getOption("tidytuesdayR.mastodon")}.}

\item{open}{Open the newly created file for editing? Happens in RStudio, if
applicable, or via \code{\link[utils:file.edit]{utils::file.edit()}} otherwise.}

\item{ignore}{Should the newly created file be added to \code{.Rbuildignore}?}
}
\value{
A logical vector indicating whether the file was created or
modified, invisibly.
}
\description{
We need a set of metadata information about each TidyTuesday dataset. Use
this function to set up the \code{meta.yaml} file for your submission (and create
the submission directory if it does not already exist). If you do not provide
values for the parameters, you will be prompted to enter them in an
interactive session.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}

  tt_meta()
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_load_gh.R
\name{tt_load_gh}
\alias{tt_load_gh}
\title{Load TidyTuesday data from Github}
\usage{
tt_load_gh(x, week = NULL, auth = gh::gh_token())
}
\arguments{
\item{x}{The date of data to pull (in "YYYY-MM-dd" format), or the four-digit
year as a number.}

\item{week}{Which week number to use within a given year. Only used when \code{x}
is a valid year.}

\item{auth}{A GitHub token. See \code{\link[gh:gh_token]{gh::gh_token()}} for more details.}
}
\value{
A \code{tt} object. This contains the files available for the week,
readme html, and the date of the TidyTuesday.
}
\description{
Pulls the readme and URLs of the data from the TidyTuesday
github folder based on the date provided
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
# check to make sure there are requests still available
if (rate_limit_check(quiet = TRUE) > 30) {
  tt_gh <- tt_load_gh("2019-01-15")
  ## readme attempts to open the readme for the weekly dataset
  readme(tt_gh)

  agencies <- tt_download(
    tt_gh,
    files = "agencies.csv"
  )
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_check_date.R
\name{tt_check_date}
\alias{tt_check_date}
\title{Generate valid TidyTuesday URL}
\usage{
tt_check_date(x, week = NULL, auth = gh::gh_token())
}
\arguments{
\item{x}{The date of data to pull (in "YYYY-MM-dd" format), or the four-digit
year as a number.}

\item{week}{Which week number to use within a given year. Only used when \code{x}
is a valid year.}

\item{auth}{A GitHub token. See \code{\link[gh:gh_token]{gh::gh_token()}} for more details.}
}
\description{
Given multiple types of inputs, generate a valid TidyTuesday URL.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/use_tidytemplate.R
\name{use_tidytemplate}
\alias{use_tidytemplate}
\title{Create and open the tidytemplate}
\usage{
use_tidytemplate(
  name = NULL,
  open = rlang::is_interactive(),
  refdate = today(),
  ignore = FALSE
)
}
\arguments{
\item{name}{A name for your generated TidyTuesday analysis Rmd, such as
"My_TidyTuesday.Rmd".}

\item{open}{Open the newly created file for editing? Happens in RStudio, if
applicable, or via \code{\link[utils:file.edit]{utils::file.edit()}} otherwise.}

\item{refdate}{Date to use as reference to determine which TidyTuesday to use
for the template. Either date object or character string in YYYY-MM-DD
format.}

\item{ignore}{Should the newly created file be added to \code{.Rbuildignore}?}
}
\value{
A logical vector indicating whether the file was created or
modified, invisibly.
}
\description{
Use the tidytemplate Rmd for starting your analysis with a leg up for
processing
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}

  use_tidytemplate(name = "My_Awesome_TidyTuesday.Rmd")
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_intro.R
\name{tt_intro}
\alias{tt_intro}
\title{Create and open intro.md}
\usage{
tt_intro(
  path = "tt_submission",
  open = rlang::is_interactive(),
  ignore = FALSE
)
}
\arguments{
\item{path}{The relative path to the directory to hold your submission files
(\code{tt_submission} by default). If this directory does not exist, it will be
created.}

\item{open}{Open the newly created file for editing? Happens in RStudio, if
applicable, or via \code{\link[utils:file.edit]{utils::file.edit()}} otherwise.}

\item{ignore}{Should the newly created file be added to \code{.Rbuildignore}?}
}
\value{
A logical vector indicating whether the file was created or
modified, invisibly.
}
\description{
When curating a TidyTuesday dataset, you need to introduce the dataset. This
function creates a simple \code{intro.md} file in the specified path (creating
that path if it does not already exist), and (if possible) opens it for
editing.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}

  tt_intro()
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_available.R
\name{tt_print}
\alias{tt_print}
\alias{print.tt_dataset_table}
\alias{print.tt_dataset_table_list}
\title{Printing Utilities for Listing Available Datasets}
\usage{
\method{print}{tt_dataset_table}(x, ..., is_interactive = interactive())

\method{print}{tt_dataset_table_list}(x, ..., is_interactive = interactive())
}
\arguments{
\item{x}{an object used to select a method.}

\item{...}{further arguments passed to or from other methods.}

\item{is_interactive}{Whether the function is being used interactively.}
}
\value{
\code{x}, invisibly
}
\description{
printing utilities for showing the available datasets for a specific year or
all time
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
# check to make sure there are requests still available
if (rate_limit_check(quiet = TRUE) > 30) {
  available_datasets_2018 <- tt_datasets(2018)
  print(available_datasets_2018)

  all_available_datasets <- tt_available()
  print(all_available_datasets)
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa-shared_params.R
\name{.shared-params}
\alias{.shared-params}
\title{Parameters used in multiple functions}
\arguments{
\item{auth}{A GitHub token. See \code{\link[gh:gh_token]{gh::gh_token()}} for more details.}

\item{call}{(\code{environment}) The caller environment for error messages.}

\item{files}{Which file names to download. Default "All" downloads all files
for the specified week.}

\item{path}{The relative path to the directory to hold your submission files
(\code{tt_submission} by default). If this directory does not exist, it will be
created.}

\item{tt}{A \code{tt} object, output from \code{\link[=tt_load_gh]{tt_load_gh()}}.}

\item{week}{Which week number to use within a given year. Only used when \code{x}
is a valid year.}

\item{x}{The date of data to pull (in "YYYY-MM-dd" format), or the four-digit
year as a number.}

\item{year}{What year of TidyTuesday to use}
}
\description{
Reused parameter definitions are gathered here for easier editing.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_load.R
\name{tt_load}
\alias{tt_load}
\title{Load TidyTuesday data from Github}
\usage{
tt_load(x, week = NULL, files = "All", ..., auth = gh::gh_token())
}
\arguments{
\item{x}{The date of data to pull (in "YYYY-MM-dd" format), or the four-digit
year as a number.}

\item{week}{Which week number to use within a given year. Only used when \code{x}
is a valid year.}

\item{files}{Which file names to download. Default "All" downloads all files
for the specified week.}

\item{...}{Additional parameters to pass to the parsing functions. Note:
These arguments will be passed for all filetypes.}

\item{auth}{A GitHub token. See \code{\link[gh:gh_token]{gh::gh_token()}} for more details.}
}
\value{
\code{tt_data} object, which contains data that can be accessed via \code{$},
and the readme for the week's TidyTuesday, which can be viewed by printing
the object or calling \code{\link[=readme]{readme()}}.
}
\description{
Load TidyTuesday data from Github
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
tt_output <- tt_load("2019-01-15")
tt_output
agencies <- tt_output$agencies
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_master_file.R
\name{should_update_tt_master_file}
\alias{should_update_tt_master_file}
\title{Decide whether to update the master file}
\usage{
should_update_tt_master_file(force = FALSE, auth = gh::gh_token())
}
\arguments{
\item{force}{force the update to occur even if the SHA matches}

\item{auth}{A GitHub token. See \code{\link[gh:gh_token]{gh::gh_token()}} for more details.}
}
\value{
Boolean indicating whether the master file should be updated.
}
\description{
Decide whether to update the master file
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_available.R
\name{tt_available}
\alias{tt_available}
\alias{tt_datasets}
\title{Listing all available TidyTuesdays}
\usage{
tt_available(auth = gh::gh_token())

tt_datasets(year, auth = gh::gh_token())
}
\arguments{
\item{auth}{A GitHub token. See \code{\link[gh:gh_token]{gh::gh_token()}} for more details.}

\item{year}{What year of TidyTuesday to use}
}
\value{
\code{tt_available()} returns a \code{tt_dataset_table_list}, which is a
list of \code{tt_dataset_table}. This class has special printing methods to show
the available data sets.

\code{tt_datasets()} returns a \code{tt_dataset_table} object. This class has
special printing methods to show the available datasets for the year.
}
\description{
The TidyTuesday project is a constantly growing repository of data sets.
Knowing what type of data is available for each week requires going to the
source. However, one of the hallmarks of 'tidytuesdayR' is that you never
have to leave your R console. These functions were
created to help maintain this philosophy.
}
\details{
To find out the available datasets for a specific year, the user
can use the function \code{tt_datasets()}. This function will either populate the
Viewer or print to console all the available data sets and the week/date
they are associated with.

To get the whole list of all the data sets ever released by TidyTuesday, the
function \code{tt_available()} was created. This function will either populate the
Viewer or print to console all the available data sets ever made for
TidyTuesday.
}
\examples{
\dontshow{if (interactive()) withAutoprint(\{ # examplesIf}
# check to make sure there are requests still available
if (rate_limit_check(quiet = TRUE) > 30) {
  ## show data available from 2018
  tt_datasets(2018)

  ## show all data available ever
  tt_available()
}
\dontshow{\}) # examplesIf}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tt_check_date.R
\name{tt_date}
\alias{tt_date}
\title{Get date of TidyTuesday, given the year and week}
\usage{
tt_date(year, week = NULL, auth = gh::gh_token())
}
\arguments{
\item{year}{What year of TidyTuesday to use}

\item{week}{Which week number to use within a given year. Only used when \code{x}
is a valid year.}

\item{auth}{A GitHub token. See \code{\link[gh:gh_token]{gh::gh_token()}} for more details.}
}
\description{
Sometimes we don't know the date we want, but we do know the week. This
function provides the ability to pass the year and week we are interested in
to get the correct date
}
\keyword{internal}
